% Pomona funkcija, ki jo uporabljajo primeri:
% Vzorceni_signali_koncni_pulzi.m
% Idealno_impulzno_vorcenje.m
% Periodicni_signali.m
% Vzorcenje_fr_spektra.m
% Periodicnost_fr_spektra.m

% Funkcija generira asovni signala, ki zadosti zahtevam teorema o vzorenju
function [t,x,fx] = Generiraj_cas_signal(n1,n2,N) 
% Vhodi:
% n1 - ni uporabljen
% 2^(N+n2) - dolina signalov na koncu funkcije
% 2^N - dolina krajih signalov
%
% Izhodi:
% t - asovna os signala
% x - asovni signal
% fx - diskretna Fourierova transformacija asovnega signala
%
% Vsi asovni signali so definirani na intervalu doline 2^(N+n2)
% Najprej generiramo redko vzoreni signal, ki ima as vzorenja 2^n2 (indeksi signalov imajo enico)
% 
% Definiramo asovno os kratkega signala
t1=0:2^n2:2^(N+n2)-1;

% Izhodini asovni signal in njegov transform
x1=t1.^2.*exp(-0.02*t1);    fx1=fft(x1);

% Poreemo visoke frekvence s Hanningovim oknom
fx1(2:end)=fx1(2:end).*(1-hanning(length(fx1(2:end))))';

% V frekvennem prostoru izvedemo subsampling, kar pomeni, da se frekvenni
% spekter dopolni z nilami v sredini, ostali del spektra pa skopiramo
fx=zeros(1,2^(N+n2));
% Prvih 2^(N-1) komponent je kopija starega spektra z indeksom 1
fx(1:2^(N-1))=fx1(1:2^(N-1));
% Zadnjih 2^(N-1) komponent je kopija starega spektra z indeksom 1
fx(end+1-2^(N-1):end)=fx1(2^(N-1)+1:end);

% Z inverzno Four. transformacijo dobimo asovni signal
x=real(ifft(fx));

% Zaradi rezanja s Hanningom se je signal malo "razlezel" v sosednjo periodo, zato ga malo premaknemo 
x=[x(end-1000:end),x(1:end-1001)];

% Izraunamo e njegovo Fourierovo transformacijo in asovno os
fx=fft(x);      t=linspace(0,2^(N+n2),2^(N+n2));
