% Ilustracija Fourierove vrste vlaka pulzov
clear all; close all

% resolucija ekrana
screen_size = get(0, 'ScreenSize');

% Perioda signala je 64 vzorcev - od tega 9 vrednosti 1 in 55 vrednosti 0
% Ker elimo, da je signal sod, mora imeti vrednosti 1 pri k = 0, +/-1, +/-2, +/-3, +/-4
% To pomeni, da je 5 enic na zaetku (prvi element vektorja za fft je k=0), 4 pa na koncu
p_osn = [ones(1,5) zeros(1,55) ones(1,4)];

% Dolina celotnega signala je 2^20 vzorcev
% Ena perioda traja 64 = 2^6 vzorcev,
% zato moramo zlepiti 2^14 period, kar lahko naredimo tako, da signal podvojimo 14-krat
for i=1:14
    p_osn=[p_osn,p_osn];
end

% definicija asovne osi
t=0:2^20-1;

% Ker bomo signal opazovali simetrino glede na as t=0,
% bomo uporabili funkcijo ifftshift za premik signala,
% od asovnega vektorja pa bomo odteli polovico trajanja (2^19)
t = t - 2^19;
plot(t,ifftshift(p_osn),'b');
axis([-300 300 -0.1 1.1]);
xlabel('$$t$$ [s]','Interpreter','Latex','FontSize',14);
ylabel('$$p_z$$','Interpreter','Latex','FontSize',14);
title('Periodi\v{c}ni signal','Interpreter','Latex','FontSize',16)
set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran

% Hitra Fourierova transformacija vlaka pulzov
fp=fft(p_osn);

% Definicija frekvencne osi
f = (0:(2^20-1))/2^20;

% Ker bomo spekter opazovali simetrino glede na as f=0,
% bomo uporabili funkcijo fftshift za premik signala,
% od asovnega vektorja pa bomo odteli polovico periode (0.5 Hz)
f = f-0.5;

% nariimo frekvenni spekter vlaka pulzov
figure;
d = 9/64;
% dodamo ovojnico sin(f) / f
ovojnica = d * sin(f*pi*9)./(f*pi*9);
plot(f,fftshift(real(fp))/2^20,'b', f,real(ovojnica), 'r')
hold on
% v ovojnici upotevamo, da je izhodini signal vzoren, zato je potrebno
% upotevati e vpliv premeenih verzij spektra (nova slika vsakih 1 Hz)
for premik = 1:10
    % upotevamo premeene verzije od -10 Hz do 10 Hz
    ovojnica = ovojnica + d * sin((f-premik)*pi*9)./((f-premik)*pi*9);
    ovojnica = ovojnica + d * sin((f+premik)*pi*9)./((f+premik)*pi*9);
end
plot(f,real(ovojnica),'g')
axis([-.5 .5 -0.04 .18]);
set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran
xlabel('$$f$$ [Hz]','Interpreter','Latex','FontSize',14);
ylabel('$$c_m$$','Interpreter','Latex','FontSize',14);
title('Koeficienti Fourierove vrste $c_m$','Interpreter','Latex','FontSize',16)
legend('c_m','ovojnica, ki bi jo dobili za zvezni periodini signal','ovojnica za diskretni signal (upoteva vpliv premeenih verzij c_m)')

text(0,0.16,['$$c_0 = \frac{A \tau}{T} = \frac{1 \cdot 9}{64} = 0$$,1406'], 'FontSize',20,'Interpreter','Latex','HorizontalAlignment','center','VerticalAlignment','middle');

%  Simbolini izraun
syms A t tau m T d
c_m = simple(1/T * int(A*exp(-j*2*pi*m/T*t),t,-tau/2,tau/2)); % definicija izrauna Fourierove vrste
c_m = simple(subs(c_m,tau,d*T)); % nadomestimo tau/T z d
figure
set(gcf,'MenuBar','none','NumberTitle','off') % odstranimo menu in tevilko grafa
axis off
text(0.5,0.5,['Simboli\v{c}ni izra\v{c}un: $$c_m=' latex(c_m) '$$'],'Interpreter','Latex','FontSize',20,'HorizontalAlignment','center','VerticalAlignment','middle')
set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran
