% Ilustracija Fourierove transformacije pulza
clear all; close all

% resolucija ekrana
screen_size = get(0, 'ScreenSize');

% Fourierovo transformacijo bomo izraunali za ozek in irok pulz
for tau = [3, 201]
    
    % Dolina celotnega signala je 2^20 vzorcev
    % Ker elimo, da je signal pulza sod, mora imeti vrednosti 1 pri k = 0, +/-1, +/-2, +/-3, +/-4 ... +/- (tau-1)/2 
    % Ostale vrednosti so enake 0
    % To pomeni, da je 5 enic na zaetku (prvi element vektorja za fft je k=0), 4 pa na koncu
    p = zeros(1,2^20);
    p(1:(tau+1)/2) = 1;
    p(end+1-(tau-1)/2:end) = 1;
    %
    % definicija asovne osi
    t=0:2^20-1;
    % Ker bomo signal opazovali simetrino glede na as t=0,
    % bomo uporabili funkcijo ifftshift za premik signala,
    % od asovnega vektorja pa bomo odteli polovico trajanja (2^19)
    t = t - 2^19;

    figure
    plot(t,ifftshift(p),'b');
    axis([-150 150 -0.1 1.1]);
    xlabel('$$t$$ [s]','Interpreter','Latex','FontSize',14);
    ylabel('$$p_z$$','Interpreter','Latex','FontSize',14);
    title(['Pulz \v{s}irine ' num2str(tau)],'Interpreter','Latex','FontSize',16)
    set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran

    % Hitra Fourierova transformacija vlaka pulzov
    fp=fft(p);
    % Fourierovo transformacijo aperiodinega signala dobimo, e pomnoimo FFT signala s periodo vzorenja T
    % (pri nas je perioda vzorenja enaka 1, tako da dobimo iste vrednosti - le enote so druge)

    % Definicija frekvenne osi
    f = (0:(2^20-1))/2^20;

    % Ker bomo spekter opazovali simetrino glede na as f=0,
    % bomo uporabili funkcijo fftshift za premik signala,
    % od asovnega vektorja pa bomo odteli polovico periode (0.5 Hz)
    f = f-0.5;

    figure;
    fp_analiticno = tau * sinc(f*tau);
    plot(f,real(fp_analiticno),'b', f,fftshift(real(fp)),'r')
    set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran
    xlabel('$$f$$ [Hz]','Interpreter','Latex','FontSize',14);
    ylabel('$$P_z(f)$$','Interpreter','Latex','FontSize',14);
    title(['Fourierova transformacija pulza \v{s}irine ' num2str(tau)],'Interpreter','Latex','FontSize',16)
    l = legend('analiti\v{c}na re\v{s}itev','re\v{s}itev, dobljena z vzor\v{c}enjem zveznega signala')
    set(l, 'Interpreter','Latex','FontSize',14)
    
    text(0,tau*1.01,['$$P_z(0) = A \tau = $$ ' num2str(tau)], 'FontSize',20,'Interpreter','Latex','HorizontalAlignment','center','VerticalAlignment','bottom');
    axis([-0.5 0.5 min(real(fp))*1.1 tau*1.1])
end

%  Simbolini izraun
syms A t f tau w
P_z = simple(fourier(A*heaviside(t+tau/2)-A*heaviside(t-tau/2)));
P_z = simple(subs(P_z,w,2*pi*f)); % nadomestimo w z 2*pi*f
figure
set(gcf,'MenuBar','none','NumberTitle','off') % odstranimo menu in tevilko grafa
axis off
text(0.5,0.5,['Simboli\v{c}ni izra\v{c}un: $$P_z(f)=' latex(P_z) '$$'],'Interpreter','Latex','FontSize',20,'HorizontalAlignment','center','VerticalAlignment','middle')
set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran
