% Ilustracija vzorenja z vlakom pulzov
% Potrebujemo e funkcijo Generiraj_cas_signal.m

clear all; close all

% resolucija ekrana
screen_size = get(0, 'ScreenSize');

% Definicija konstant za realizacijo vlaka pulzov
n1=3;  % 2^n1 = irina pulza (tau)
n2=6;  % 2^n2 = dolina periode vlaka pulzov (T)
N=13;  % 2^N  = tevilo period
% Celoten signal ima dolino 2^N * 2^n2
tau = 2^n1;
T = 2^n2;

% Generiramo asovni signal
[t,x,fx] = Generiraj_cas_signal([],n2,N); 

% Generiramo vlak pulzov (perioda je 2^n2, dolina pulza 2^n1)
p(1:2^n2)=0;  p(1:tau)=1;
% Nato naredimo 2^N period
for i=1:N
    p=[p,p];
end
% Ker morajo pulzi imeti povrino 1, mora biti amplituda pulzov 1/tau
p = p * 1/tau;

% Izraunamo FFT vlaka pulzov
fp=fft(p);

% Vzoreni asovni signal
x_zvezda=x.*p;

% Nariimo prvih 2000 sek asovnega signala
% Primerjamo zvezni signal in vzoreni signal (slednji mora piti pomnoen s
% tau, ker je zadeva normirana na povrino pulza 1)
figure; plot(t,x,'r:', t,tau*x_zvezda,'b'); axis([0 2000 -1 25])
xlabel('$$t$$ [s]','Interpreter','Latex','FontSize',14);
ylabel('$$x_z(t), \tau x^*(t)$$','Interpreter','Latex','FontSize',14);
title('Zvezni in vzor\v{c}eni signal','Interpreter','Latex','FontSize',16)
l = legend('$$x_z(t)$$', '$$\tau x^*(t)$$');
set(l, 'Interpreter','Latex','FontSize',14)
set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran

% Izraunamo FFT vzorenega asovnega signala, ki ima enako tevilsko
% vrednost kot Fourierova transformacija (perioda vzorenja je namre 1 sek)
fx_zvezda=fft(x_zvezda);

% Definicija frekvenne osi
f = (0:(2^(N+n2)-1))/2^(N+n2);

% Ker bomo spekter opazovali simetrino glede na as f=0,
% bomo uporabili funkcijo fftshift za premik signala,
% od asovnega vektorja pa bomo odteli polovico periode (0.5 Hz)
f = f-0.5;

% Nariimo frekvenni spekter originalnega (rde) in vorenega (moder) asovnega signala
figure
plot(f,T*fftshift(abs(fx_zvezda)),'b', f, fftshift(abs(fx)),'r:')
axis([-0.5 0.5 -100 4000])
xlabel('$$f$$ [Hz]','Interpreter','Latex','FontSize',14);
ylabel('$$|X_z(f)|, T |X^*(f)|$$','Interpreter','Latex','FontSize',14);
title('Spekter zveznega in vzor\v{c}enega signala','Interpreter','Latex','FontSize',16)
l = legend('$$T |X^*(f)|$$', '$$|X_z(f)|$$');
set(l, 'Interpreter','Latex','FontSize',14)

set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran

