% Ilustracija WhittakerShannonove interpolacijske formule

close all
clear all

% resolucija ekrana
screen_size = get(0, 'ScreenSize');

% Line width
lw_narrow = .5;
lw_wide = 2.5;

% Colors
cb = [0 0 0]; % black color
cw = [1 1 1]; % black color
cbl = [0 0 1]; % blue
cr = [1 0 0]; % red

t = -4:0.01:4; % signal izrisujemo na intervalu [-4,4]
x_z = 2*sin(pi*t); % zvezni signal

f_s = 4; % Frekvenca vzorenja 4 Hz
T = 1/f_s; % cas vzorcenja je 0,25
t_v = -4:T:4; % kvantizirani as
x_v = 2*sin(pi*t_v); % vzoreni signal

figure

set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran

handli = []; % vektor roic prejnjih funkcij rekonstruiranih signalov

% Izris zveznega in vzorenega signala ter oznaba osi
line(t,x_z, 'LineWidth', lw_wide, 'Color', cbl, 'LineStyle', '-','Clipping','off');
hold on
line(t_v,x_v, 'Color', cb, 'LineStyle', 'none','Clipping','off','Marker','o','MarkerSize',6,'MarkerFaceColor',cw);
xlabel('$$t$$','Interpreter','Latex','FontSize',12)
set(gca,'YLim',[-2.3 3.25])

% Obvestilo na vrhu
h_text = text(0,2.8,'Rde\v{c}i signal je rekonstrukcija modrega iz vzorcev od $k=0$ do $k=0$','FontSize',20,'Interpreter','Latex','HorizontalAlignment','center','VerticalAlignment','bottom');
h_obvest = text(0,2.4,'Za nadaljevanje pritisnite katerokoli tipko','FontSize',20,'Interpreter','Latex','HorizontalAlignment','center','VerticalAlignment','bottom','Color',[0 0.5 0]);

% Zanka za rekonstrukcijo, ki poteka od k=-odmik do k=+odmik
for odmik = 0:16
    indeks_srednjega_Matlab = floor(length(t_v)/2) + 1;
    x_r_odmik = x_v(indeks_srednjega_Matlab) * sinc(f_s*t); % rekonstrukcija samo z vzorcem pri k=0
    for k=1:odmik
        x_r_odmik = x_r_odmik + x_v(indeks_srednjega_Matlab+k) * sinc(f_s*(t-k*T));     % rekonstrukcija samo z vzorcem pri k
        x_r_odmik = x_r_odmik + x_v(indeks_srednjega_Matlab-k) * sinc(f_s*(t-(-k)*T));  % rekonstrukcija samo z vzorcem pri -k
    end
    h1 = line(t,x_r_odmik, 'LineWidth', lw_narrow, 'LineStyle','-', 'Color',cr, 'LineWidth', lw_wide);
    set(h_text,'String',['Rde\v{c}i signal je rekonstrukcija modrega iz vzorcev od $k=' num2str(-odmik) '$ do $k=' num2str(odmik) '$'])
    pause
    %uiwait(msgbox(sprintf('Za nadaljevanje\nklikni OK ali pritisni ENTER'),'','modal'));
    set(h1, 'Color', 1-cw*(0.04+0.06*odmik), 'LineWidth', lw_narrow); %barvo stare rekonstrukcije spremenimo v sivo (svetli odtenki - iz malega tevila vzorcev, temni - iz velikega tevila)
    handli = [handli h1];
end

delete(handli) % Izbriemo stare rekonstrukcije
delete(h_obvest)
line(t,x_r_odmik, 'LineWidth', lw_narrow, 'LineStyle','-', 'Color',cr, 'LineWidth', lw_wide);
