% Ilustracija oversamplinga pri CDjih
% Potrebujemo e zvoni zapis c1.wav

clear all;close all

% resolucija ekrana
screen_size = get(0, 'ScreenSize');

% Definicija parametrov
N_resample = 16; % Najprej signal resamplamo oz. decimiramo (obdrimo vsak 16. vzorec)
N_sinx = 20;     % N_sinx = stevilo period funkcije sint/t

% Preberemo signal in vzamemo prvih 2^19 vzorcev ter normiramo na interval [-1, 1]
[x_original,Fs_orig,Nbits]=wavread('c1');
x_original=x_original(1:2^19); x_original=x_original/max(abs(x_original));
T_orig = 1/Fs_orig;
T_resample = N_resample * T_orig;
Fs_resample = Fs_orig / N_resample;

% definiramo asovno os
t=0:T_orig:(length(x_original)-1)/Fs_orig;

figure(4)
set(gcf,'MenuBar','none','NumberTitle','off')
axis([0 1 0 1]);
h1 = text(.5,.55,'Predvajam originalni signal.','FontSize',20,'Interpreter','Latex','HorizontalAlignment','center','VerticalAlignment','middle');
text(.5,.45,'Po\v{c}akajte, prosim.','FontSize',20,'Interpreter','Latex','HorizontalAlignment','center','VerticalAlignment','middle','Color', [0 0.5 0]);
axis off
drawnow
% predvajamo original
wavplay(x_original,Fs_orig);

% Decimiramo signal
x_decimiran= x_original(1:N_resample:end);

% Generiramo signal iz zadrevalnika 0-tega reda (vsak vzorec 16x razmnoimo)
x_ZOH=ones(N_resample,1)*x_decimiran';
x_ZOH=reshape(x_ZOH,1,[]);

% Predvajamo signal z zadrevalnikom 0-tega reda
set(h1,'String','Predvajam signal (rekonstrukcija z ZOH).');
drawnow
wavplay(x_ZOH,Fs_orig);

% Generiramo decimirani vzoreni signal na asovni osi originalnega signala
% (vmesni vzorci so enaki 0)
x_sampl=[x_decimiran';zeros(N_resample-1,length(x_decimiran))];
x_sampl=reshape(x_sampl,1,[]);

% Impulzni odziv idealnega nizkega filtra
% asovna os N_sinx resamplanih period naprej in nazaj
tt = -N_sinx*T_resample:T_orig:N_sinx*T_resample;
ideal_nizki_filt=sinc(Fs_resample*tt);
% nariimo odziv idealnega nizkega filtra
figure
plot(tt,ideal_nizki_filt);
xlabel('$$t$$ [s]','Interpreter','Latex','FontSize',14);
ylabel('$$g_{ideal}(t)$$','Interpreter','Latex','FontSize',14);
title('Impulzni odziv rekonstrukcijksega filtra','Interpreter','Latex','FontSize',16)
set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran

% Interpolacija z idealnim nizkim filtrom
oversamp_signal=conv(x_sampl,ideal_nizki_filt);
N_izpustni_vzorce = floor(length(tt)/2); % tevilo izpuenih vzorcev zaradi conv
oversamp_signal = oversamp_signal(N_izpustni_vzorce+1:end-N_izpustni_vzorce);

% Predvajamo rekonstruirani signal
figure(4)
set(h1,'String','Predvajam signal (idealna rekonstrukcija).');
drawnow
wavplay(oversamp_signal,Fs_orig);
close(4)

% Nariimo vse signale; 
N_izpustni_vzorce = floor(length(tt)/2); % tevilo izpuenih vzorcev
figure
plot(t,x_original,'b', t,oversamp_signal,'r', t,x_ZOH,'g', t(1:N_resample:end),x_sampl(1:N_resample:end),'b*')
xlabel('$$t$$ [s]','Interpreter','Latex','FontSize',14);
ylabel('$$x(t)$$','Interpreter','Latex','FontSize',14);
title('Originalni in dva rekonstruirana signala','Interpreter','Latex','FontSize',16)
l = legend('original', 'rekonstrukcija (ideal)', 'rekonstrukcija (ZOH)');
set(l, 'Interpreter','Latex','FontSize',14)
set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran

% e enkrat nariemo detajl
figure
plot(t,x_original,'b', t,oversamp_signal,'r', t,x_ZOH,'g', t(1:N_resample:end),x_sampl(1:N_resample:end),'b*')
xlabel('$$t$$ [s]','Interpreter','Latex','FontSize',14);
ylabel('$$x(t)$$','Interpreter','Latex','FontSize',14);
title('Originalni in dva rekonstruirana signala','Interpreter','Latex','FontSize',16)
l = legend('original', 'rekonstrukcija (ideal)', 'rekonstrukcija (ZOH)');
set(l, 'Interpreter','Latex','FontSize',14)
axis([5.69 5.72 -.25 .25])
set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran
