% Ilustracija fenomena, da ima periodinost asovnega signala za posledico diskretnost frekvennega signala
% Potrebujemo e funkcijo Generiraj_cas_signal.m

clear all; close all

% resolucija ekrana
screen_size = get(0, 'ScreenSize');

% Definicija konstant za realizacijo signala
n2=6;  % 2^n2 = tevilo period
N=13;  % 2^N  = dolina periode periodinega signala (T)
T = 2^N;
% Celoten signal ima dolino 2^N * 2^n2

% Generiramo asovni signal
[t,x,fx]=Generiraj_cas_signal([],n2,N); 

% Naredimo asovni signal periodien
x_per=x(1:2^N); % od celotnega aperiodinega signala vzamemo dolino ene periode periodilnega signala
for i=1:n2
    x_per=[x_per,x_per]; % sestavimo 2^n2 period
end

% Nariimo periodini in aperiodini signal
figure
plot(t,x_per,'b', t,x,'r:')
xlabel('$$t$$ [s]','Interpreter','Latex','FontSize',14);
ylabel('$$x_z(t), x_p(t)$$','Interpreter','Latex','FontSize',14);
title('Zvezni in periodi\v{c}ni signal','Interpreter','Latex','FontSize',16)
l = legend('$$x_p(t)$$', '$$x_z(t)$$');
set(l, 'Interpreter','Latex','FontSize',14)
axis([0 30000 -1 18])

set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran

% Izraun FFT originalnega signala
fx=fft(x);
% Ker je perioda vzorenja 1, je to enako Fourierovi transformaciji

% Izraun FFT periodinega
fx_per=fft(x_per);
% X_p oz. koeficiente Fourierove vrste dobimo po deljenju s tevilom vzorcev
fx_per = fx_per / length(fx_per);

% Definicija frekvenne osi
f = (0:(2^(N+n2)-1))/2^(N+n2);

% Ker bomo spekter opazovali simetrino glede na as f=0,
% bomo uporabili funkcijo fftshift za premik signala,
% od asovnega vektorja pa bomo odteli polovico periode (0.5 Hz)
f = f-0.5;

% Nariimo frekvenni spekter periodicnega in neperiodinega signala
figure
plot(f,T*fftshift(abs(fx_per)),'b', f, fftshift(abs(fx)),'r:')
axis([-0.01 0.01 -100 4000])
xlabel('$$f$$ [Hz]','Interpreter','Latex','FontSize',14);
ylabel('$$|X_z(f)|, T |X_p(mF)|$$','Interpreter','Latex','FontSize',14);
title('Spekter zveznega in periodi\v{c}nega signala','Interpreter','Latex','FontSize',16)
l = legend('$$T |X_p(mF)|$$', '$$|X_z(f)|$$');
set(l, 'Interpreter','Latex','FontSize',14)

set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran
