% Ilustracija fenomena, da ima vzorenje (diskretnost) frekvennega signala za posledico periodinost asovnega signala
% Potrebujemo e funkcijo Generiraj_cas_signal.m

clear all; close all

% resolucija ekrana
screen_size = get(0, 'ScreenSize');

% Definicija konstant za realizacijo signala
% Zanima nas le rezultat fx, ker tokrat izhajamo iz frekvennega prostora
n2=6;  % 2^n2 = tevilo period
N=13;  % 2^N  = dolina periode periodinega signala (T)
T = 2^N;
% Celoten signal ima dolino 2^N * 2^n2

% Generiramo asovni signal
[t,x,fx]=Generiraj_cas_signal([],n2,N); 

% Definicija frekvenne osi
f = (0:(2^(N+n2)-1))/2^(N+n2);

% Ker bomo spekter opazovali simetrino glede na as f=0,
% bomo uporabili funkcijo fftshift za premik signala,
% od asovnega vektorja pa bomo odteli polovico periode (0.5 Hz)
f = f-0.5;

% Vlak pulzov za vzorenje frekvennega spektra
fp = zeros(size(fx));
fp(1:2^n2:end)=1; % perioda vzorenja v frekv. prostoru je 2^n2 (F)
F = 2^n2;

% Vzorimo frekvenni spekter
% Vzoreni spekter matematino zapiemo kot X^{f*}(f)
% (podobno smo pisali vzoreni asovni signal x^*(t))
% "f*" v eksponentu pomeni, da vzorimo v frekvennem prostoru
% "*" v eksponentu je pomenila, da vzorimo v asovnem prostoru
f_zvezda=fx.*fp; 
figure
% Nariimo vzoreni in nevzoreni frekvenni spekter
plot(f,fftshift(abs(f_zvezda)),'b', f, fftshift(abs(fx)),'r:')
axis([-0.01 0.01 -100 4000])
xlabel('$$f$$ [Hz]','Interpreter','Latex','FontSize',14);
ylabel('$$|X_z(f)|, |X^{f*}(f)|$$','Interpreter','Latex','FontSize',14);
title('Spekter zveznega signala in vzor\v{c}eni spekter zveznega signala','Interpreter','Latex','FontSize',16)
l = legend('$$|X^{f*}(f)|$$', '$$|X_z(f)|$$');
set(l, 'Interpreter','Latex','FontSize',14)

set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran

% Ker je perioda vzorenja 1, je Fourierova transformacija
% zveznega signala (tudi vzorenega) enaka pripadajoi FFT,
% zato lahko asovni odziv dobimo z IFFT
% Naredimo inverzno Fourierovo transformacijo frekvennega spektra
x_f_zvezda=real(ifft(f_zvezda));
% Dualno s problemom vzorenja asovnega signala x^*(t), kjer je veljalo:
% X^*(f) = 1/T * X(f)
% kjer je T perioda vzorenja v asovnem prostoru
% dobimo za rezultat inverzne transformacije vzorenega spektra X^{f*}(f):
% x^{f*}(t) = 1/F * x(t)
% kjer je F korak vzorenja v frekvennem prostoru 2^n2

% Spremenljivka x_f_zvezda je torej F-krat manja od zveznega signala

figure
% Narisimo asovna signala
plot(t,F*x_f_zvezda,'b', t,x,'r:') % signal, dobljen iz vzorenega spektra, je popravljen s faktorjem F
xlabel('$$t$$ [s]','Interpreter','Latex','FontSize',14);
ylabel('$$x_z(t), F x^{f*}(t)$$','Interpreter','Latex','FontSize',14);
title('Zvezni signal in signal, dobljen z inverzno Fourierovo transformacijo vzor\v{c}enega spektra (pomno\v{z}en s $F$)','Interpreter','Latex','FontSize',16)
l = legend('$$F x^{f*}(t)$$', '$$x_z(t)$$');
set(l, 'Interpreter','Latex','FontSize',14)
axis([0 30000 -1 18])

set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran
