% Ilustracija fenomena, da ima periodinost frekvennega spektra za posledico diskretnost zveznega signala
% Potrebujemo e funkcijo Generiraj_cas_signal.m

clear all; close all

% resolucija ekrana
screen_size = get(0, 'ScreenSize');

% Definicija konstant za realizacijo signala
% Zanima nas le rezultat fx, ker tokrat izhajamo iz frekvennega prostora
n2=6;  % 2^n2 = tevilo period
N=13;  % 2^N  = dolina periode periodinega signala (T)
% Celoten signal ima dolino 2^N * 2^n2

% Generiramo asovni signal
[t,x,fx]=Generiraj_cas_signal([],n2,N); 

% Definicija frekvenne osi
f = (0:(2^(N+n2)-1))/2^(N+n2);

% Ker bomo spekter opazovali simetrino glede na as f=0,
% bomo uporabili funkcijo fftshift za premik signala,
% od asovnega vektorja pa bomo odteli polovico periode (0.5 Hz)
f = f-0.5;

% Naredimo frekvenni spekter periodien
% Na celotnem intervalu 1 Hz bomo imeli 2^5=32 period
% Vsaka perioda v frekvennem prostoru ima 2^14 elementov
n_per = 5;    % 2^n_per = tevilo period v frekvennem prostoru
F_perioda = 1 / 2^n_per; % dolina periode (1/32 Hz)
n_v_per = 14; % 2^n_v_per = tevilo vzorcev v periodi
fp=[fx(1:2^(n_v_per-1)),fx(end-2^(n_v_per-1)+1:end)];
for i=1:n_per
    fp=[fp,fp];
end
% Periodini spekter matematino zapiemo kot X_{fp}(f)
% (podobno smo pisali periodini asovni signal x_p(t))
% indeks "fp" pomeni, da je periodinost v frekvennem prostoru
% indeks "p" je pomenil, da je periodinost v asovnem prostoru

% Nariimo frekvenna spektra
figure
plot(f,fftshift(abs(fp)),'b', f, fftshift(abs(fx)),'r:')
axis([-0.5 0.5 -100 4000])
xlabel('$$f$$ [Hz]','Interpreter','Latex','FontSize',14);
ylabel('$$|X_z(f)|, |X_{fp}(f)|$$','Interpreter','Latex','FontSize',14);
title('Spekter zveznega signala in periodi\v{c}no razmno\v{z}en spekter','Interpreter','Latex','FontSize',16)
l = legend('$$|X_{fp}(f)|$$', '$$|X_z(f)|$$');
set(l, 'Interpreter','Latex','FontSize',14)

set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran

% Ker je perioda vzorenja 1, je Fourierova transformacija
% zveznega signala enaka pripadajoi FFT, zato lahko asovni odziv dobimo z IFFT
% Naredimo inverzno Fourierovo transformacijo periodinega spektra
x_v=real(ifft(fp));

figure
plot(t,F_perioda*x_v,'b', t,x,'r:')
xlabel('$$t$$ [s]','Interpreter','Latex','FontSize',14);
ylabel('$$x_z(t), F_p x_{fp}(t)$$','Interpreter','Latex','FontSize',14);
title('Zvezni signal in signal, dobljen z inverzno Fourierovo transformacijo periodi\v{c}nega spektra (pomno\v{z}en s $F_p$)','Interpreter','Latex','FontSize',16)
l = legend('$$F_p x_{fp}(t)$$', '$$x_z(t)$$');
set(l, 'Interpreter','Latex','FontSize',14)
axis([0 2000 -1 18])

set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran

