% Ilustracija kompresije podatkov s pomojo DFT
% Potrebujemo e zvoni zapis c1.wav

clear all;close all

% resolucija ekrana
screen_size = get(0, 'ScreenSize');

% Preberemo signal in vzamemo prvih 2^19 vzorcev, odstranimo srednjo vrednost ter normiramo na interval [-1, 1]
[x,Fs,Nbits]=wavread('c1');
x=x(1:2^19);
x=x-mean(x);
x=x/max(abs(x));

% definiramo asovno os
t=0:1/Fs:(length(x)-1)/Fs;

fx=fft(x);          % FFT signala
afx=abs(fx);        % abs. vrednost FFT
afx_s=sort(afx);    % sortiranje po abs. vrednosti FFT (od najmanjega do najvejega)
afx_s=flipud(afx_s);% naredimo flip gor-dol - sortiranje je od najvejega do najmanjega

% Uporabnik pove, koliko % komponent eli obdrati v spektru, vse ostale postanejo 0
prag_string = inputdlg('Koliko % najvejih komponent FFT elite obdrati?','',1,{'10'});
prag_stevilo = str2num(prag_string{1}); % tevilska vrednost praga
indeks = round(prag_stevilo*length(x)/100); % poie indeks mejnega lena v sortirani vrsti

% Vse elemente, ki so manji od afx_s(indeks), postavimo na 0
% Ostali elementi ostanejo nespremenjeni:
% mult(elementa) = 0, e afx(elementa) < afx_s(indeks)
% mult(elementa) = 1, e afx(elementa) > afx_s(indeks)
mult=(1+sign(afx-afx_s(indeks)+eps))/2;
fx1=fx.*mult; % fx seveda ni sortiran, tudi mult ni bil narejen na sortiranemu
% fx1 ima torej le e doloen % komponent FFT, ki so razline od 0

% Izraun Inverzne transformacije
x1=real(ifft(fx1));
x1=x1/max(abs(x1));

% Kvantizirali bomo spekter, in sicer posebej realni del in posebej imaginarni del
% Uporabnik pove, koliko bo tevilo bitov kvantizacije 
Nbit_string = inputdlg('Kakno tevilo bitov kvantizacije boste uporabili?','',1,{'6'});
Nbit_kvant = str2num(Nbit_string{1}); % tevilska vrednost praga

fx1_real = real(fx1); % realni del spektra
fx1_imag = imag(fx1); % imaginarni del spektra
q_real = (max(fx1_real)-min(fx1_real))/2^Nbit_kvant;
q_imag = (max(fx1_imag)-min(fx1_imag))/2^Nbit_kvant;
fx2_real = quant(fx1_real,q_real);
fx2_imag = quant(fx1_imag,q_imag);
fx2 = fx2_real + j*fx2_imag;

% Izraun Inverzne transformacije
x2=real(ifft(fx2));
x2=x2/max(abs(x2));

figure(4)
set(gcf,'MenuBar','none','NumberTitle','off')
axis([0 1 0 1]);
h1 = text(.5,.55,['Predvajam signal, ki vsebuje ' prag_string{1} '\% komponent.'],'FontSize',20,'Interpreter','Latex','HorizontalAlignment','center','VerticalAlignment','middle');
text(.5,.45,'Po\v{c}akajte, prosim.','FontSize',20,'Interpreter','Latex','HorizontalAlignment','center','VerticalAlignment','middle','Color', [0 0.5 0]);
axis off
drawnow
% predvajamo signal, ki ima doloen % komponent enak 0
wavplay(x1,Fs);

set(h1,'String','Predvajam prej\v{s}nji signal, ki ima \v{s}e kvantiziran spekter.');
drawnow
% predvajamo signal s kvantiziranim spektrom
wavplay(x2,Fs);

close all

figure
plot(fx,'b.')
hold on
plot(fx1,'r.')
plot(fx2,'g.')
xlabel('Re(fft(x))','Interpreter','Latex','FontSize',14);
ylabel('Im(fft(x))','Interpreter','Latex','FontSize',14);
title('Originalni spekter in dva spektra kompresiranega signala','Interpreter','Latex','FontSize',16)
l = legend('original', 'samo postavitev na 0', 'kvantizacija spektra');
set(l, 'Interpreter','Latex','FontSize',14)
set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran

figure
plot(fx,'b.')
hold on
plot(fx1,'r.')
plot(fx2,'g.')
xlabel('Re(fft(x))','Interpreter','Latex','FontSize',14);
ylabel('Im(fft(x))','Interpreter','Latex','FontSize',14);
title('Originalni spekter in dva spektra kompresiranega signala (detajl)','Interpreter','Latex','FontSize',16)
l = legend('original', 'samo postavitev na 0', 'kvantizacija spektra');
set(l, 'Interpreter','Latex','FontSize',14)
axis(afx_s(indeks)*1.5*[-1 1 -1 1])
set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran

figure
plot(t,x,'b',t,x1,'r',t,x2,'g')
xlabel('$$t$$ [s]','Interpreter','Latex','FontSize',14);
ylabel('$$x(t)$$','Interpreter','Latex','FontSize',14);
title('Originalni signal in dva kompresiranega signala','Interpreter','Latex','FontSize',16)
l = legend('original', 'samo postavitev na 0', 'kvantizacija spektra');
set(l, 'Interpreter','Latex','FontSize',14)
set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran

figure
plot(t,x,'b',t,x1,'r',t,x2,'g')
xlabel('$$t$$ [s]','Interpreter','Latex','FontSize',14);
ylabel('$$x(t)$$','Interpreter','Latex','FontSize',14);
title('Originalni signal in dva kompresiranega signala (detajl)','Interpreter','Latex','FontSize',16)
l = legend('original', 'samo postavitev na 0', 'kvantizacija spektra');
set(l, 'Interpreter','Latex','FontSize',14)
axis([5.69 5.72 -.25 .25])
set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran
