% Ilustracija konvolucije dveh signalov kot odziva sistema

clear all
close all

% resolucija ekrana
screen_size = get(0, 'ScreenSize');

% Prvi sistem G(z) = 0.4*z/(z-0.6)
% Sistem ima takojnji odziv na vzbujanje
Stevec = [0.4 0];
Imenovalec = [1 -0.6];

k_os = 0:30; % asovna os

figure
set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran

u = zeros(31,1); u(6:16) = 1; % vzbujalni signal je impulz

% Na prvem grafu je vzbujalni signal u(k)
subplot(321);
stem(k_os,u); % vzbujalni signal
axis([0 30 -1 1]);
xlabel('$$k$$','Interpreter','Latex','FontSize',12);
ylabel('$$u(k)$$','Interpreter','Latex','FontSize',12);

% Na drugem grafu je vzbujalni impulzni odziv h(k)
h = dimpulse(Stevec,Imenovalec,k_os); % impulzni odziv sistema Stevec(z)/Imenovalec(z) na asovnem intervalu k_os
subplot(322);
stem(k_os,h);
axis([0 30 -1 1]);
xlabel('$$k$$','Interpreter','Latex','FontSize',12);
ylabel('$$h(k)$$','Interpreter','Latex','FontSize',12);

suma = zeros(31,1); % vektor za sprotno setevanje vrednosti signala (s konvolucijo)
for m = 0:30
    % Na tretjem grafu je trenutni (ob asu m) vzorec vzbzujalnega signala,
    % ki ga interpretiramo kot zakasnjen delta impulz z amplitudo
    % trenutnega vzorca: u(m) delta(k-m)
    subplot(323);
    stem(m,u(m+1),'r'); % u(m+1) zaradi indeksiranja signala v Matlabu
    axis([0 30 -1 1]);
    xlabel('$$k$$','Interpreter','Latex','FontSize',12);
    ylabel(['$$u(',num2str(m),') \delta(k-',num2str(m),')$$'],'Interpreter','Latex','FontSize',12)
    
    % Na etrtem grafu je odziv na impulz na tretjem grafu: u(m) h(k-m)
    % Najprej je v signalu m nicel, potem pa z u(m) pomnoen impulzni odziv
    umhkm = [zeros(m,1); u(m+1) * h(1:31-m)]; % u(m+1) zaradi indeksiranja signala v Matlabu
    subplot(324);
    stem(k_os,umhkm,'r');
    axis([0 30 -1 1]);
    xlabel('$$k$$','Interpreter','Latex','FontSize',12);
    ylabel(['$$u(',num2str(m),') h(k-',num2str(m),')$$'],'Interpreter','Latex','FontSize',12)

    % Na petem grafu je setevek vseh dosedanjih odzivov na grafu 4
    % To je konvolucijska vsota, ki se sproti seteva
    suma=suma+umhkm;
    subplot(325);
    stem(k_os,suma);
    axis([0 30 -1 1]);
    xlabel('$$k$$','Interpreter','Latex','FontSize',12);
    ylabel(['$$\sum_{m=0}^{',num2str(m),'} u(m) h(k-m)$$'],'Interpreter','Latex','FontSize',12)
    
    % Na estem grafu so vsi vzorci y(k), ki so e dokonni - to so vzorci od k = 0 do k = m
    subplot(326);
    stem(k_os(1:m+1),suma(1:m+1)); % (m+1) zaradi indeksiranja signala v Matlabu
    axis([0 30 -1 1]);
    xlabel('$$k$$','Interpreter','Latex','FontSize',12);
    ylabel(['$$y(k), 0 \leq k \leq ',num2str(m),'$$'],'Interpreter','Latex','FontSize',12)

    if m == 0
        subplot(321)
        h_obvest = text(25,1.2,'Za nadaljevanje pritisnite katerokoli tipko','FontSize',20,'Interpreter','Latex','HorizontalAlignment','left','VerticalAlignment','bottom','Color',[0 0.5 0]);
    end
    pause
end

delete(h_obvest);

% Drugi sistem G(z) = 0.4/(z-0.6)
% Sistem nima takojnjega odziva na vzbujanje (ekvivalent zveznega sistema 1. reda)
Stevec = [0.4];
Imenovalec = [1 -0.6];

k_os = 0:30; % asovna os

figure
set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran

u = zeros(31,1); u(6:16) = 1; % vzbujalni signal je impulz

% Na prvem grafu je vzbujalni signal u(k)
subplot(321);
stem(k_os,u); % vzbujalni signal
axis([0 30 -1 1]);
xlabel('$$k$$','Interpreter','Latex','FontSize',12);
ylabel('$$u(k)$$','Interpreter','Latex','FontSize',12);

% Na drugem grafu je vzbujalni impulzni odziv h(k)
h = dimpulse(Stevec,Imenovalec,k_os); % impulzni odziv sistema Stevec(z)/Imenovalec(z) na asovnem intervalu k_os
subplot(322);
stem(k_os,h);
axis([0 30 -1 1]);
xlabel('$$k$$','Interpreter','Latex','FontSize',12);
ylabel('$$h(k)$$','Interpreter','Latex','FontSize',12);

suma = zeros(31,1); % vektor za sprotno setevanje vrednosti signala (s konvolucijo)
for m = 0:30
    % Na tretjem grafu je trenutni (ob asu m) vzorec vzbzujalnega signala,
    % ki ga interpretiramo kot zakasnjen delta impulz z amplitudo
    % trenutnega vzorca: u(m) delta(k-m)
    subplot(323);
    stem(m,u(m+1),'r'); % u(m+1) zaradi indeksiranja signala v Matlabu
    axis([0 30 -1 1]);
    xlabel('$$k$$','Interpreter','Latex','FontSize',12);
    ylabel(['$$u(',num2str(m),') \delta(k-',num2str(m),')$$'],'Interpreter','Latex','FontSize',12)
    
    % Na etrtem grafu je odziv na impulz na tretjem grafu: u(m) h(k-m)
    % Najprej je v signalu m nicel, potem pa z u(m) pomnoen impulzni odziv
    umhkm = [zeros(m,1); u(m+1) * h(1:31-m)]; % u(m+1) zaradi indeksiranja signala v Matlabu
    subplot(324);
    stem(k_os,umhkm,'r');
    axis([0 30 -1 1]);
    xlabel('$$k$$','Interpreter','Latex','FontSize',12);
    ylabel(['$$u(',num2str(m),') h(k-',num2str(m),')$$'],'Interpreter','Latex','FontSize',12)

    % Na petem grafu je setevek vseh dosedanjih odzivov na grafu 4
    % To je konvolucijska vsota, ki se sproti seteva
    suma=suma+umhkm;
    subplot(325);
    stem(k_os,suma);
    axis([0 30 -1 1]);
    xlabel('$$k$$','Interpreter','Latex','FontSize',12);
    ylabel(['$$\sum_{m=0}^{',num2str(m),'} u(m) h(k-m)$$'],'Interpreter','Latex','FontSize',12)
    
    % Na estem grafu so vsi vzorci y(k), ki so e dokonni - to so vzorci od k = 0 do k = m
    subplot(326);
    stem(k_os(1:m+1),suma(1:m+1)); % (m+1) zaradi indeksiranja signala v Matlabu
    axis([0 30 -1 1]);
    xlabel('$$k$$','Interpreter','Latex','FontSize',12);
    ylabel(['$$y(k), 0 \leq k \leq ',num2str(m),'$$'],'Interpreter','Latex','FontSize',12)

    if m == 0
        subplot(321)
        h_obvest = text(25,1.2,'Za nadaljevanje pritisnite katerokoli tipko','FontSize',20,'Interpreter','Latex','HorizontalAlignment','left','VerticalAlignment','bottom','Color',[0 0.5 0]);
    end
    pause
end

delete(h_obvest);
