% Raunanje frekvennega odziva po definiciji ter z uporabo Control toolbox (bode) in Signal processing Toobox (freqz)
%
%                    -1   
%               1 + z   
% sistem G(z) = -------
%                 2

close all
clear all

% resolucija ekrana
screen_size = get(0, 'ScreenSize');

% Line width
lw_narrow = .5;
lw_wide = 2.5;

% Colors
cb = [0 0 0]; % black color
cw = [1 1 1]; % black color
cbl = [0 0 1]; % blue
cr = [1 0 0]; % red
cg = [0 0.6 0]; % green

% Definiramo frekvenno os (za krono frekvenco) od 0 do polovice frekvence vzorenja
omega_T=linspace(0,pi,50);

% Izraunamo frekvenni odziv
z=exp(j*omega_T);
H=(1+z.^-1)/2;
% in ga nariemo v linearnem merilu na prvi graf
roc1=figure;
set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran
subplot(211)
plot(omega_T,abs(H))
title('Frekven\v{c}ni odziv $G(z)=\frac{1+z^{-1}}{2}$ v linearnem merilu','Interpreter','Latex','FontSize',16)
xlabel('$$\omega$$','Interpreter','Latex','FontSize',14);
ylabel('$$|G(e^{j\omega T})|$$','Interpreter','Latex','FontSize',14);
axis([0 3.2 -.03 1.03])
subplot(212)
plot(omega_T,angle(H)*180/pi)
xlabel('$$\omega$$','Interpreter','Latex','FontSize',14);
ylabel('$$\angle G(e^{j\omega T})$$','Interpreter','Latex','FontSize',14);
axis([0 3.2 -100 10])

% Isto naredimo v logaritemskem merilu na drugi graf
% Da bodo toke na sliki bolj enakomerno izbrane, mora biti tudi frekvenni vektor logaritemsko sestavljen
omega_T_log=logspace(-2,log10(pi),50);
z=exp(j*omega_T_log);
H_log=(1+z.^-1)/2;
figure
set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran
subplot(211)
loglog(omega_T_log,abs(H_log))
title('Frekven\v{c}ni odziv $G(z)=\frac{1+z^{-1}}{2}$ v logaritemskem merilu','Interpreter','Latex','FontSize',16)
xlabel('$$\omega$$','Interpreter','Latex','FontSize',14);
ylabel('$$|G(e^{j\omega T})|$$','Interpreter','Latex','FontSize',14);
axis([0 3.2 0.001 1.1])
subplot(212)
semilogx(omega_T_log,angle(H_log)*180/pi)
xlabel('$$\omega$$','Interpreter','Latex','FontSize',14);
ylabel('$$\angle G(e^{j\omega T})$$','Interpreter','Latex','FontSize',14);
axis([0 3.2 -100 10])

% 2. metoda - freqz iz Signal processing toolboxa (na prvi graf)

%omega_T_2=linspace(0,pi,20);
Hfreqz=freqz(0.5*[1 1],[1 0],omega_T);
% nariemo na sliko 1
figure(roc1)
subplot(211)
line(omega_T,abs(Hfreqz),'Color',cr,'LineStyle','none','Marker','o')
subplot(212)
line(omega_T,angle(Hfreqz)*180/pi,'Color',cr,'LineStyle','none','Marker','o')

% 3. mtoda - bode iz Control toolboxa (na prvi graf)

% Bode - control TB
sys=tf(0.5*[1 1],[1 0],1);
[ampl,faza]=bode(sys,omega_T);
ampl=ampl(:); % iz 3-D arraya naredimo stolpni vektor (reshape)
faza=faza(:); % iz 3-D arraya naredimo stolpni vektor (reshape)
% nariemo na sliko 1
figure(roc1)
subplot(211)
line(omega_T,ampl,'Color',cg,'LineStyle','none','Marker','+')
legend('definicija','freqz','bode')
subplot(212)
line(omega_T,faza,'Color',cg,'LineStyle','none','Marker','+')
legend('definicija','freqz','bode')

% Izris Nyquistovega diagrama na 3. graf
figure;
% 1. nain - po definiciji (iz na zaetku izraunanega H)
plot(real(H),imag(H))
% 2. nain - freqz (iz Hfreqz)
line(real(H),imag(H),'Color',cr,'LineStyle','none','Marker','o')
% 3. nain
[re,im]=nyquist(sys,omega_T);
re=re(:); % iz 3-D arraya naredimo stolpni vektor (reshape)
im=im(:); % iz 3-D arraya naredimo stolpni vektor (reshape)
line(re,im,'Color',cg,'LineStyle','none','Marker','+')
 
axis equal
title('Nyquistov diagram $G(z)=\frac{1+z^{-1}}{2}$','Interpreter','Latex','FontSize',16)
xlabel('Re $$[G(e^{j\omega T})]$$','Interpreter','Latex','FontSize',14);
ylabel('Im $$[G(e^{j\omega T})]$$','Interpreter','Latex','FontSize',14);
legend('definicija','freqz','bode')
set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran



