% Ilustracija preslikave imaginarne osi kot meje leve in desne polravnine
% po treh metodah preslikave (prvih diferenc, zadnjih diferenc in Tustinovega pravila) 

close all;   clear all;

% resolucija ekrana
screen_size = get(0, 'ScreenSize');

% Odpremo sliko
figure
set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran
hold on
axis([-2 2 -2 2]); axis equal

% Definiramo zvezno frekvenco
omegaT = -100:.1:100;
sT = j*omegaT;             % gosta os za risanje zveznih krivulj
omegaT_redek = -100:1:100; % redka os za risanje kricev
sT_redek = j*omegaT_redek;

xx = cos(-pi:pi/180:pi);
yy = sin(-pi:pi/180:pi);
hkrog = patch(xx,yy,[1 0.85 0.85]); % enotni krog je ronat
set(hkrog,'EdgeColor','none');

% Definiramo prvo preslikavo (Prve diference)
z1 = 1+sT;
z1_redek = 1+sT_redek;

% Definiramo drugo preslikavo (Zadnje diference)
z2 = 1./(1-sT);
z2_redek = 1./(1-sT_redek);

% Definiramo tretjo preslikavo (Tustin)  
z3 = (1 +sT/2)./(1-sT/2);
z3_redek = (1 +sT_redek/2)./(1-sT_redek/2);

h1 = plot(real(z1),imag(z1),'b',real(z2),imag(z2),'r',real(z3),imag(z3),'g',real(z1_redek),imag(z1_redek),'bx',real(z2_redek),imag(z2_redek),'rx',real(z3_redek),imag(z3_redek),'gx');
set(h1,'LineWidth',2,'MarkerSize',10)
l = legend([h1(1:3);hkrog], 'Prve dif.','Zadnje dif.','Tustin','enotski krog');
set(l,'FontSize',12)
xlabel('Re $$[z]$$','Interpreter','Latex','FontSize',14);
ylabel('Im $$[z]$$','Interpreter','Latex','FontSize',14);
title('Preslikava imaginarne osi v ravnini $s$: $s=j \omega$','Interpreter','Latex','FontSize',16)
