% 

close all
clear all

% resolucija ekrana
screen_size = get(0, 'ScreenSize');

% Line width
lw_narrow = 0.4;
lw_wide = 2;

% Colors
cb = [0 0 0]; % black color
cw = [1 1 1]; % white color
cr = [1 0 0]; % red color
cbl = [0 0 1]; % blue color
cg = [0 0.5 0]; % green color
clightr = [1 0.85 0.85]; % light red color
cgrey = [0.5 0.5 0.5]; % grey color

T1 = 0.4;                    % as vzorenja sistema H1(z)
T2 = 4;                      % as vzorenja sistema H2(z)
s = tf('s');                 % operator s kot objekta tipa LTI
Gz = 1 / (s^2 + 0.1*s + 1);  % zvezna prenosna funkcija
H1 = c2d(Gz, T1, 'tustin');  % diskretizirana pren. f. s asom vzor. T1
H2 = c2d(Gz, T2, 'tustin');  % diskretizirana pren. f. s asom vzor. T2

bodeplot(Gz, 'bo', logspace(-1,1,601))
hold on
bodeplot(H1, 'rx', logspace(-1,log10(pi/T1)-1e-7,601))
bodeplot(H2, 'g*', logspace(-1,log10(pi/T2)-1e-7,601))

l = legend('$G_z(s)$','$H_1(z)$','$H_2(z)$','Location','SouthWest');
set(l, 'interpreter','latex');

set(0,'ShowHiddenHandles','on')


h = findobj(gcf,'String','Frequency  (rad/sec)');
set(h,'String','$\omega$','interpreter','latex','FontSize',12);

h = findobj(gcf,'String','Magnitude (dB)');
set(h,'String','${\cal A}$ [dB]','interpreter','latex','FontSize',12);

% os za prvi graf Bodeja
axes(get(h,'Parent'))
axis([.1 10 -50 25])

h = findobj(gcf,'String','Phase (deg)');
set(h,'String','${\cal B}$ [$^{\circ}$]','interpreter','latex','FontSize',12);

% os za drugi graf Bodeja
axes(get(h,'Parent'))
axis([.1 10  -190 10])

sive_crte = findobj(gcf,'type','line','LineStyle','-','Color',[0 0 0]);
set(sive_crte,'LineWidth',lw_narrow,'Marker','none','LineStyle','-','Color',cgrey);

crte = findobj(gcf,'type','line','Marker','o');
set(crte,'LineWidth',lw_wide,'Marker','none','LineStyle','-','Color',cbl);
crte = findobj(gcf,'type','line','Marker','x');
set(crte,'LineWidth',lw_wide,'Marker','none','LineStyle','-','Color',cr);
crte = findobj(gcf,'type','line','Marker','*');
set(crte,'LineWidth',lw_wide,'Marker','none','LineStyle','-','Color',cg);

set(0,'ShowHiddenHandles','off')

set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran


% simbolina reitev
clear all

T1 = sym(0.4); % simbolini zapis desetikega tevila asa vzorenja T1
T2 = sym(4);   % simbolini zapis desetikega tevila asa vzorenja T2
syms s z       % simbolini zapis spremenljivk s in z
Gz = 1 / (s^2 + 0.1*s + 1);       % zvezna prenosna funkcija
H1 = subs(Gz,s,2/T1*(z-1)/(z+1)); % Tustinovo pravilo za vzorenje T1
pretty(simple(H1))                % "lep" zapis poenostavljenga izraza
H2 = subs(Gz,s,2/T2*(z-1)/(z+1)); % Tustinovo pravilo za vzorenje T2
pretty(simple(H2))                % "lep" zapis poenostavljenga izraza




