clear all
close all

% resolucija ekrana
screen_size = get(0, 'ScreenSize');


%% Optimizacija ojaenja P-regulatorja za proces prvega reda

G = zpk([],[0.9],.2,-1);

J1v = [];
J2v = [];
J3v = [];
J4v = [];

KV = 0.1:0.1:10; % vektor ojaenj, ki jih bomo preizkusili
kv = [0:1000]';  % asovni vektor posameznega simulacijskega teka
for K = KV
    G_zz = minreal(K*G/(1+K*G));
    y = step(G_zz, kv);
    e = 1-y;
    J1v = [J1v sum(e)/length(e)];
    J2v = [J2v sum(e.^2)/length(e)];
	J3v = [J3v sum(abs(e))/length(e)];
	J4v = [J4v sum(kv.*abs(e))/length(e)];
end

figure
set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran
plot(KV, J1v)
axis([0 10 0 min(J1v)*10])
xlabel('$$K$$','Interpreter','Latex','FontSize',14);
ylabel('$$J_1$$','Interpreter','Latex','FontSize',14);
title('$G_P(z)=\frac{0.2}{z-0.9},G_R(z)=K,J_1=\lim_{N \rightarrow \infty} \frac{1}{N+1} \sum_{k=0}^{N} e(k)$','Interpreter','Latex','FontSize',16)

figure
set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran
plot(KV, J2v)
axis([0 10 0 min(J2v)*10])
xlabel('$$K$$','Interpreter','Latex','FontSize',14);
ylabel('$$J_2$$','Interpreter','Latex','FontSize',14);
title('$G_P(z)=\frac{0.2}{z-0.9},G_R(z)=K,J_2=\lim_{N \rightarrow \infty} \frac{1}{N+1} \sum_{k=0}^{N} e^2(k)$','Interpreter','Latex','FontSize',16)

figure
set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran
plot(KV, J3v)
axis([0 10 0 min(J3v)*10])
xlabel('$$K$$','Interpreter','Latex','FontSize',14);
ylabel('$$J_3$$','Interpreter','Latex','FontSize',14);
title('$G_P(z)=\frac{0.2}{z-0.9},G_R(z)=K,J_3=\lim_{N \rightarrow \infty} \frac{1}{N+1} \sum_{k=0}^{N} |e(k)|$','Interpreter','Latex','FontSize',16)

figure
set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran
plot(KV, J4v)
axis([0 10 0 min(J4v)*10])
xlabel('$$K$$','Interpreter','Latex','FontSize',14);
ylabel('$$J_4$$','Interpreter','Latex','FontSize',14);
title('$G_P(z)=\frac{0.2}{z-0.9},G_R(z)=K,J_4=\lim_{N \rightarrow \infty} \frac{1}{N+1} \sum_{k=0}^{N} k|e(k)|$','Interpreter','Latex','FontSize',16)

figure
set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran
plot(KV, J1v/min(J1v), KV, J2v/min(J2v), KV, J3v/min(J3v), KV, J4v/min(J4v))
axis([0 10 0 10])
xlabel('$$K$$','Interpreter','Latex','FontSize',14);
ylabel('$$\frac{J_i}{\min(J_i)}$$','Interpreter','Latex','FontSize',14);
title('Potek normiranih kriterijskih funkcij $\frac{J_i}{\min(J_i)}$','Interpreter','Latex','FontSize',16)
l = legend('$\frac{J_1}{\min(J_1)}$','$\frac{J_2}{\min(J_2)}$','$\frac{J_3}{\min(J_3)}$','$\frac{J_4}{\min(J_4)}$');
set(l, 'Interpreter','Latex','FontSize',14);


% 2. primer
% proces tretjega reda (3 zvezni poli in 0 zveznih niel) in PID-regulator

G_z = tf(0.1, [1 2 2 1]);
G = c2d(G_z, 0.1);

s = tf('s');

%% Optimizacija ojaenja P-lena

J1v = [];
J2v = [];
J3v = [];
J4v = [];
KV = 1:.2:30; % vektor ojaenj P-regulatorja, ki jih bomo preizkusili
kv = [0:1000]';  % asovni vektor spremenljivke k posameznega simulacijskega teka
for K = KV
    K_P = K;
    K_I = 0;
    T_D = 0;
    K_R = K_P + K_I/s + K_P*T_D/(1+s*0.1*T_D);
    K_R = c2d(K_R, 0.1, 'tustin');
    G_zz = minreal(K_R*G/(1+K_R*G));
    y = step(G_zz, kv*0.1);
    e = 1-y;
    J1v = [J1v sum(e)/length(e)];
    J2v = [J2v sum(e.^2)/length(e)];
	J3v = [J3v sum(abs(e))/length(e)];
	J4v = [J4v sum(kv.*abs(e))/length(e)];
end

figure
set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran
plot(KV, J1v)
xlabel('$$K_P$$','Interpreter','Latex','FontSize',14);
ylabel('$$J_1$$','Interpreter','Latex','FontSize',14);
title('$G_P(z)=\frac{0.2}{z-0.9},G_R(z)=K,J_1=\lim_{N \rightarrow \infty} \frac{1}{N+1} \sum_{k=0}^{N} e(k)$','Interpreter','Latex','FontSize',16)

figure
set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran
plot(KV, J2v)
axis([0 30 0 min(J2v)*10])
xlabel('$$K_P$$','Interpreter','Latex','FontSize',14);
ylabel('$$J_2$$','Interpreter','Latex','FontSize',14);
title('$G_P(z)=\frac{0.2}{z-0.9},G_R(z)=K,J_2=\lim_{N \rightarrow \infty} \frac{1}{N+1} \sum_{k=0}^{N} e^2(k)$','Interpreter','Latex','FontSize',16)

figure
set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran
plot(KV, J3v)
axis([0 30 0 min(J3v)*10])
xlabel('$$K_P$$','Interpreter','Latex','FontSize',14);
ylabel('$$J_3$$','Interpreter','Latex','FontSize',14);
title('$G_P(z)=\frac{0.2}{z-0.9},G_R(z)=K,J_3=\lim_{N \rightarrow \infty} \frac{1}{N+1} \sum_{k=0}^{N} |e(k)|$','Interpreter','Latex','FontSize',16)

figure
set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran
plot(KV, J4v)
axis([0 30 0 min(J4v)*10])
xlabel('$$K_P$$','Interpreter','Latex','FontSize',14);
ylabel('$$J_4$$','Interpreter','Latex','FontSize',14);
title('$G_P(z)=\frac{0.2}{z-0.9},G_R(z)=K,J_4=\lim_{N \rightarrow \infty} \frac{1}{N+1} \sum_{k=0}^{N} k|e(k)|$','Interpreter','Latex','FontSize',16)

figure
set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran
plot(KV, J2v/min(J2v), KV, J3v/min(J3v), KV, J4v/min(J4v))
axis([0 30 0 10])
xlabel('$$K_P$$','Interpreter','Latex','FontSize',14);
ylabel('$$\frac{J_i}{\min(J_i)}$$','Interpreter','Latex','FontSize',14);
title('Potek normiranih kriterijskih funkcij $\frac{J_i}{\min(J_i)},T_I = \infty,T_D=0$','Interpreter','Latex','FontSize',16)
l = legend('$\frac{J_2}{\min(J_2)}$','$\frac{J_3}{\min(J_3)}$','$\frac{J_4}{\min(J_4)}$');
set(l, 'Interpreter','Latex','FontSize',14);


%% Optimizacija integrirne asovne konstante
J1v = [];
J2v = [];
J3v = [];
J4v = [];
KV = 5:1:100; % vektor integrirnih asovnih konstant T_I, ki jih bomo preizkusili
kv = [0:5000]';  % asovni vektor spremenljivke k posameznega simulacijskega teka
for K = KV
    T_I = K;
    K_P = 25;
    K_I = K_P/T_I;
    T_D = 0;
    K_R = K_P + K_I/s + K_P*T_D/(1+s*0.1*T_D);
    K_R = c2d(K_R, 0.1, 'tustin');
    G_zz = minreal(K_R*G/(1+K_R*G));
    y = step(G_zz, kv*0.1);
    e = 1-y;
    J2v = [J2v sum(e.^2)/length(e)];
	J3v = [J3v sum(abs(e))/length(e)];
	J4v = [J4v sum(kv.*abs(e))/length(e)];
end
figure
set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran
plot(KV, J2v/min(J2v), KV, J3v/min(J3v), KV, J4v/min(J4v))
axis([0 100 0 10])
xlabel('$$T_I$$','Interpreter','Latex','FontSize',14);
ylabel('$$\frac{J_i}{\min(J_i)}$$','Interpreter','Latex','FontSize',14);
title('Potek normiranih kriterijskih funkcij $\frac{J_i}{\min(J_i)},K_P = 25,T_D = 0$','Interpreter','Latex','FontSize',16)
l = legend('$\frac{J_2}{\min(J_2)}$','$\frac{J_3}{\min(J_3)}$','$\frac{J_4}{\min(J_4)}$');
set(l, 'Interpreter','Latex','FontSize',14);

%% Optimizacija diferencirne asovne konstante
J1v = [];
J2v = [];
J3v = [];
J4v = [];
KV = .02:.02:2; % vektor integrirnih asovnih konstant T_D, ki jih bomo preizkusili
kv = [0:5000]';  % asovni vektor spremenljivke k posameznega simulacijskega teka
for K = KV
    T_D = K;
    K_P = 25;
    T_I = 45;
    K_I = K_P/T_I;
    K_R = K_P + K_I/s + K_P*T_D*s/(1+s*0.1*T_D);
    K_R = c2d(K_R, 0.1, 'tustin');
    G_zz = minreal(K_R*G/(1+K_R*G));
    y = step(G_zz, kv*0.1);
    e = 1-y;
    J2v = [J2v sum(e.^2)/length(e)];
	J3v = [J3v sum(abs(e))/length(e)];
	J4v = [J4v sum(kv.*abs(e))/length(e)];
end
figure
set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran
plot(KV, J2v/min(J2v), KV, J3v/min(J3v), KV, J4v/min(J4v))
axis([0 2 0 10])
xlabel('$$T_D$$','Interpreter','Latex','FontSize',14);
ylabel('$$\frac{J_i}{\min(J_i)}$$','Interpreter','Latex','FontSize',14);
title('Potek normiranih kriterijskih funkcij $\frac{J_i}{\min(J_i)},K_P = 25,T_I = 45$','Interpreter','Latex','FontSize',16)
l = legend('$\frac{J_2}{\min(J_2)}$','$\frac{J_3}{\min(J_3)}$','$\frac{J_4}{\min(J_4)}$');
set(l, 'Interpreter','Latex','FontSize',14);
