%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% diskretizacija PID regulatorja
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clc
clear all
close all

% resolucija ekrana
screen_size = get(0, 'ScreenSize');

syms s z K_P T_I T_D T

% Metoda zadnjih diferenc
disp('Metoda zadnjih diferenc')
s=1/T*(z-1)/z;
Gpid=K_P*(1+1/(T_I*s)+T_D*s);     % zvezna prenosna funkcija PID-a

[n,d]=numden(Gpid);
n = collect(simple(n),z);
d = collect(simple(d),z);

n_koef = coeffs(n,z) % koeficienti rastejo od nije k viji potenci
d_koef = coeffs(d,z) % koeficienti rastejo od nije k viji potenci

n_koef = simple(n_koef/d_koef(end));
d_koef = simple(d_koef/d_koef(end));

n_koef = fliplr(n_koef);
d_koef = [fliplr(d_koef) 0]

q_0 = simple(n_koef(1))
q_1 = simple(n_koef(2))
q_2 = simple(n_koef(3))

% Reitev (zadnje diference) - rahlo preoblikovana:
% u(k) = u(k-1) + q0*e(k) + q1*e(k-1) + q2*e(k-2)
%
% q_0 =  K_P * (T_D/T + T/T_I + 1)
% q_1 = -K_P * (1 + 2*T_D/T)
% q_2 =  K_P * T_D/T


% Tustinovo pravilo (oziroma ekvivalentno trapezno pravilo)
% To pomeni, da smo integriranje izvedli numerino po trapezni metodi

disp('Tustinovo pravilo')
s=2/T*(z-1)/(z+1);     

Gpid=K_P*(1+1/(T_I*s)+T_D*s);     % zvezna prenosna funkcija PID-a

[n,d]=numden(Gpid);
n = collect(simple(n),z);
d = collect(simple(d),z);

n_koef = coeffs(n,z) % koeficienti rastejo od nije k viji potenci
d_koef = coeffs(d,z) % koeficienti rastejo od nije k viji potenci

n_koef = simple(n_koef/d_koef(end));
d_koef = simple(d_koef/d_koef(end));

n_koef = fliplr(n_koef);
d_koef = [d_koef(2) 0 d_koef(1)]

q_0 = simple(n_koef(1))
q_1 = simple(n_koef(2))
q_2 = simple(n_koef(3))

% Reitev (Tustinovo pravilo) - rahlo preoblikovana:
% u(k) = u(k-2) + q0*e(k) + q1*e(k-1) + q2*e(k-2)
%
% q_0 =  K_P * (1 + T/(2*T_I) + (2*T_D)/T)
% q_1 = -K_P * (4*T_D/T - T/T_I)
% q_2 =  K_P * (-1 + T/(2*T_I) + 2*T_D/T)


disp('Tustinovo pravilo realni PID')
s=2/T*(z-1)/(z+1);     

syms T_f C_f C_I C_D
Gpid=K_P*(1+1/(T_I*s)+T_D*s/(1+T_f*s));     % zvezna prenosna funkcija PID-a

[n,d]=numden(Gpid);
n = collect(simple(n),z);
d = collect(simple(d),z);

n_koef = coeffs(n,z) % koeficienti rastejo od nije k viji potenci
d_koef = coeffs(d,z) % koeficienti rastejo od nije k viji potenci

n_koef = fliplr(simple(n_koef/d_koef(end)));
d_koef = fliplr(simple(d_koef/d_koef(end)));

disp('Vsi imenovalci spodnjih konstant so (1+2*C_f)')
p_1 = simple(-d_koef(2));
p_1 = subs(p_1, {T_f,T_D,T_I},{C_f*T,C_D*T,T/C_I});
p_1_stevec = simple(p_1*(1+2*C_f))
p_2 = simple(-d_koef(3));
p_2 = subs(p_2, {T_f,T_D,T_I},{C_f*T,C_D*T,T/C_I});
p_2_stevec = simple(p_2*(1+2*C_f))
q_0 = simple(n_koef(1));
q_0 = subs(q_0, {T_f,T_D,T_I},{C_f*T,C_D*T,T/C_I});
q_0_stevec_K_P = simple(q_0*(1+2*C_f)/K_P)
q_1 = simple(n_koef(2));
q_1 = subs(q_1, {T_f,T_D,T_I},{C_f*T,C_D*T,T/C_I});
q_1_stevec_minus_K_P = simple(q_1*(1+2*C_f)/(-K_P))
q_2 = simple(n_koef(3));
q_2 = subs(q_2, {T_f,T_D,T_I},{C_f*T,C_D*T,T/C_I});
q_2_stevec_K_P = simple(q_2*(1+2*C_f)/K_P)



% PRAKTIEN PRIMER:
%   G_z(s) = 10/(s+1)(s+2)(s+3)
G_z = zpk([],[-1 -2 -3],10)
K_KR = 6; % mejna stabilnost; za test poskusite: step(minreal(K_KR*G_z/(1+K_KR*G_z))) 
% PID nihajni preizkus
w_KR = sqrt(11); % krona frekvenca nihanja
T_KR = 2*pi/w_KR;
K_P = 0.6*K_KR;
T_I = 0.5*T_KR;
T_D = 0.125*T_KR;
s = tf('s'); % s je operator s (Control Toolbox)
G_PID_id = K_P*(1+1/(T_I*s)+T_D*s); % zvezni idealni PID
G_PID_real = K_P*(1+1/(T_I*s)+T_D*s/(1+T_D*0.2*s)); % zvezni realni PID (vzeli smo drastino dolino asovne konstante filtra D-lena: T_f = 0.3*T_D)

T_1 = 0.01; % Kratek as vzorenja
G_T_1 = c2d(G_z, T_1); % ZOH diskretizacija
z = tf('z',T_1); % z je operator z (Control Toolbox)
% zadnje diference
T = T_1;
q_0 =  K_P * (T_D/T + T/T_I + 1);
q_1 = -K_P * (1 + 2*T_D/T);
q_2 =  K_P * T_D/T;
G_PID_ZD_T1 = (q_0*z^2 + q_1*z + q_2) / (z^2-z);
% Tustin
q_0 =  K_P * (1 + T/(2*T_I) + (2*T_D)/T);
q_1 = -K_P * (4*T_D/T - T/T_I);
q_2 =  K_P * (-1 + T/(2*T_I) + 2*T_D/T);
G_PID_Tus_T1 = (q_0*z^2 + q_1*z + q_2) / (z^2-1);

T_2 = 0.1; % Dolg as vzorenja
G_T_2 = c2d(G_z, T_2); % ZOH diskretizacija
z = tf('z',T_2); % z je operator z (Control Toolbox)
% zadnje diference
T = T_2;
q_0 =  K_P * (T_D/T + T/T_I + 1);
q_1 = -K_P * (1 + 2*T_D/T);
q_2 =  K_P * T_D/T;
G_PID_ZD_T2 = (q_0*z^2 + q_1*z + q_2) / (z^2-z);
% Tustin
q_0 =  K_P * (1 + T/(2*T_I) + (2*T_D)/T);
q_1 = -K_P * (4*T_D/T - T/T_I);
q_2 =  K_P * (-1 + T/(2*T_I) + 2*T_D/T);
G_PID_Tus_T2 = (q_0*z^2 + q_1*z + q_2) / (z^2-1);

G_PID_Tus_T2_real = c2d(G_PID_real, T_2, 'Tustin');

figure
set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran

[y_z_id,t_id] = step(minreal(G_PID_id*G_z/(1+G_PID_id*G_z)),12);
plot(t_id, y_z_id,'b')
hold on

[y_z_real,t_real] = step(minreal(G_PID_real*G_z/(1+G_PID_real*G_z)),12);
plot(t_real, y_z_real,'r')

[y_ZD_T1,t_ZD_T1] = step(minreal(G_PID_ZD_T1*G_T_1/(1+G_PID_ZD_T1*G_T_1)),12);
plot(t_ZD_T1, y_ZD_T1,'g')

[y_Tus_T1,t_Tus_T1] = step(minreal(G_PID_Tus_T1*G_T_1/(1+G_PID_Tus_T1*G_T_1)),12);
plot(t_Tus_T1, y_Tus_T1,'m')

G_zz_ZD_T2 = minreal(G_PID_ZD_T2*G_T_2/(1+G_PID_ZD_T2*G_T_2));
%[y_ZD_T2,t_ZD_T2] = step(G_zz_ZD_T2,12);
%plot(t_ZD_T2, y_ZD_T2,'c+')
G_zz_ZD_T2_resample = d2d(G_zz_ZD_T2,T_1,'zoh'); % s tem resamplamo na hitreji as vzorenja - za "zvezno" simulacijo
[y_ZD_T2_resample,t_ZD_T2_resample] = step(G_zz_ZD_T2_resample,12);
plot(t_ZD_T2_resample, y_ZD_T2_resample,'c')

G_zz_Tus_T2 = minreal(G_PID_Tus_T2*G_T_2/(1+G_PID_Tus_T2*G_T_2));
%[y_Tus_T2,t_Tus_T2] = step(G_zz_Tus_T2,12);
%plot(t_Tus_T2, y_Tus_T2,'k+')
G_zz_Tus_T2_resample = d2d(G_zz_Tus_T2,T_1,'zoh'); % s tem resamplamo na hitreji as vzorenja - za "zvezno" simulacijo
[y_Tus_T2_resample,t_Tus_T2_resample] = step(G_zz_Tus_T2_resample,12);
plot(t_Tus_T2_resample, y_Tus_T2_resample,'k')

G_zz_Tus_T2_real = minreal(G_PID_Tus_T2_real*G_T_2/(1+G_PID_Tus_T2_real*G_T_2));
G_zz_Tus_T2_real_resample = d2d(G_zz_Tus_T2_real,T_1,'zoh'); % s tem resamplamo na hitreji as vzorenja - za "zvezno" simulacijo
[y_Tus_T2_real_resample,t_Tus_T2_real_resample] = step(G_zz_Tus_T2_real_resample,12);
hh = plot(t_Tus_T2_real_resample, y_Tus_T2_real_resample,'y');
set(hh,'Color',[1 0.6 0]); % rumena je presvetla za prikaz

axis([0 12 0 1.7])
xlabel('$$t$$','Interpreter','Latex','FontSize',16);
ylabel('$$y_z$$','Interpreter','Latex','FontSize',16);
legend('zvezni PID_{id}','zvezni PID_{real}','disk PID_{zd1}','disk PID_{Tus1}','disk PID_{zd2}','disk PID_{Tus2}','disk PID_{Tus2real}')

% Obvestilo na vrhu
ylim = get(gca,'YLim');
h_obvest = text(6,ylim(1)+1.02*(ylim(2)-ylim(1)),'Poglejmo si prehodni pojav od blizu. Pritisnite katerokoli tipko.','FontSize',20,'Interpreter','Latex','HorizontalAlignment','center','VerticalAlignment','bottom','Color',[0 0.5 0]);
pause
axis([0.8 6 .6 1.7])

ylim = get(gca,'YLim');
set(h_obvest, 'String','Za ogled regulirnih signalov pritisnite katerokoli tipko.','Position',[3.4 ylim(1)+1.02*(ylim(2)-ylim(1)) 0]);
pause
delete(h_obvest)


figure
set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran

[u_z_id,t_id] = step((1/(0.001*s+1))*minreal(G_PID_id/(1+G_PID_id*G_z)),12); % da lahko sploh izvedem simulacijo moram odziv (delta-impulz) filtrirati skozi hitri nizkoprepustni filter
plot(t_id, u_z_id,'b')
hold on

[u_z_real,t_real] = step(minreal(G_PID_real/(1+G_PID_real*G_z)),12);
plot(t_real, u_z_real,'r')

[u_ZD_T1,t_ZD_T1] = step(minreal(G_PID_ZD_T1/(1+G_PID_ZD_T1*G_T_1)),12);
stairs(t_ZD_T1, u_ZD_T1,'g')

[u_Tus_T1,t_Tus_T1] = step(minreal(G_PID_Tus_T1/(1+G_PID_Tus_T1*G_T_1)),12);
h1 = stairs(t_Tus_T1, u_Tus_T1,'m');

[u_ZD_T2,t_ZD_T2] = step(minreal(G_PID_ZD_T2/(1+G_PID_ZD_T2*G_T_2)),12);
stairs(t_ZD_T2, u_ZD_T2,'c')

[u_Tus_T2,t_Tus_T2] = step(minreal(G_PID_Tus_T2/(1+G_PID_Tus_T2*G_T_2)),12);
stairs(t_Tus_T2, u_Tus_T2,'k')

[u_Tus_T2_real,t_Tus_T2_real] = step(minreal(G_PID_Tus_T2_real/(1+G_PID_Tus_T2_real*G_T_2)),12);
hh = stairs(t_Tus_T2_real, u_Tus_T2_real,'y');
set(hh,'Color',[1 0.6 0]); % rumena je presvetla za prikaz

xlabel('$$t$$','Interpreter','Latex','FontSize',16);
ylabel('$$u_z$$','Interpreter','Latex','FontSize',16);
legend('zvezni PID_{id}','zvezni PID_{real}','disk PID_{zd1}','disk PID_{Tus1}','disk PID_{zd2}','disk PID_{Tus2}','disk PID_{Tus2real}')

% Obvestilo na vrhu
ylim = get(gca,'YLim');
h_obvest = text(6,ylim(1)+1.02*(ylim(2)-ylim(1)),'Pokukajmo v prvo sekundo. Pritisnite katerokoli tipko.','FontSize',20,'Interpreter','Latex','HorizontalAlignment','center','VerticalAlignment','bottom','Color',[0 0.5 0]);
pause
axis([0 1 -200 600])

ylim = get(gca,'YLim');
set(h_obvest, 'String','Za izbris Tustina pri \v{c}asu vzor\v{c}enja $T_1$ pritisnite katerokoli tipko.','Position',[0.5 ylim(1)+1.02*(ylim(2)-ylim(1)) 0]);
pause
delete(h1)
axis([0 1 -18 24])

ylim = get(gca,'YLim');
set(h_obvest, 'String','Poglejmo si sliko \v{s}e pobli\v{z}e. Pritisnite katerokoli tipko.','Position',[0.5 ylim(1)+1.02*(ylim(2)-ylim(1)) 0]);
pause
axis([0 2 -4 10])

ylim = get(gca,'YLim');
set(h_obvest, 'String','Kako pa izgleda celoten prehodni pojav? Pritisnite katerokoli tipko.','Position',[1 ylim(1)+1.02*(ylim(2)-ylim(1)) 0]);
pause
axis([0 12 -18 24])

ylim = get(gca,'YLim');
set(h_obvest, 'String','Za ogled Bodejevih diagramov regulatorjev pritisnite katerokoli tipko.','Position',[6 ylim(1)+1.02*(ylim(2)-ylim(1)) 0]);
pause
delete(h_obvest)

figure
set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran

bode(G_PID_id,G_PID_real,G_PID_ZD_T1,G_PID_Tus_T1,G_PID_ZD_T2,G_PID_Tus_T2,G_PID_Tus_T2_real)
legend('zvezni PID_{id}','zvezni PID_{real}','disk PID_{zd1}','disk PID_{Tus1}','disk PID_{zd2}','disk PID_{Tus2}','disk PID_{Tus2real}')
