% Nartovanje splonega linearnega regulatorja za pozicioniranje polov

clc
clear all
close all

syms z k p1 q0 q1

B = .05*z^(-1);
A = ((1-0.9*z^(-1))*(1-0.95*z^(-1)));

% zakasnitve ni, d = 0
% stopnja A(z^-1) in B(z^-1) je m = 2, eprav je b_2 = 0
P = 1+p1*z^(-1);  % stopnja(P) = mi = m+d-1 = 1
Q = q0+q1*z^(-1); % stopnja(A) = ni = m-1 = 1

PAQB = simple(P*A + Q*B);
A_zeleni  = (1 - 0.8*z^(-1))^3;  % stopnja(A_zeleni) = 2m+d-1 = 3

enacba = PAQB-A_zeleni; % ta enaba mora veljati za vse vrednosti z; ker iem tri neznanke, vstavim poljubne 3 neznanke, za katere enaba mora veljati
enacba1 = subs(enacba,z,1);
enacba2 = subs(enacba,z,2);
enacba3 = subs(enacba,z,3);
% imam 3 enabe za tri neznanke (p1,q0,q1)
% teava bi lahko bila le, e bi bile enabe linearno odvisne
[pp1,qq0,qq1]=solve(enacba1,enacba2,enacba3,p1,q0,q1); % reim tri enabe na tri neznanke


P0 = subs(P,{p1,q0,q1},{pp1,qq0,qq1}); % vstavim reitev v P in dobim P0
Q0 = subs(Q,{p1,q0,q1},{pp1,qq0,qq1}); % vstavim reitev v Q in dobim Q0

G_P = B/A;     % proces
G_R = Q0/P0;   % regulator (reitev problema pozicionirananja polov)

G_w = simple(G_P*G_R/(1+G_P*G_R)); % prenosna funkcija z reference na regulirano veliino
G_u = simple(G_R/(1+G_P*G_R));     % prenosna funkcija z reference na regulirno veliino

poli_w = eval(solve(1/G_w))          % poli funkcije G_w
nicle_w = eval(solve(G_w))           % nile funkcije G_w
poli_u = eval(solve(1/G_u))          % poli funkcije G_u
nicle_u = eval(solve(G_u))           % nile funkcije G_u

odziv_na_step = iztrans(G_w * z/(z-1),z,k);  % odziv na stopnico
odziv_regulirne = iztrans(G_u * z/(z-1),z,k);  % odziv na stopnico

k_v = 0:40;
figure
subplot(211)
stem(k_v, subs(odziv_na_step,k,k_v))
xlabel('$$k$$','Interpreter','Latex','FontSize',14);
ylabel('$$y(k)$$','Interpreter','Latex','FontSize',14);
subplot(212)
stairs(k_v, subs(odziv_regulirne,k,k_v))
xlabel('$$k$$','Interpreter','Latex','FontSize',14);
ylabel('$$u(k)$$','Interpreter','Latex','FontSize',14);


syms z k p1 q0 q1

B = .05*z^(-1);
A = ((1-0.9*z^(-1))*(1-0.95*z^(-1)));

% HOEM E INTEGRIRNO AKCIJO

syms p2 q2

P = 1 + p1*z^(-1) + p2*z^(-2);  % stopnja(P) = mi = m+d = 2
Q = q0 + q1*z^(-1) + q2*z^(-2); % stopnja(A) = ni = m = 2

PAQB = simple(P*A + Q*B);
A_zeleni  = (1 - 0.8*z^(-1))^4;  % stopnja(A_zeleni) = 2m+d = 4

enacba = PAQB-A_zeleni; % ta enaba mora veljati za vse vrednosti z;
% ker iem pet neznank, potrebujem 5 enab - 4 dobim tako, da vstavim poljubne 4 neznanke, za katere enaba mora veljati
% peto enabo dobim tako, da upotevam zahtevo za I-regulator: sum(p_i) = -1
enacba1 = subs(enacba,z,1);
enacba2 = subs(enacba,z,2);
enacba3 = subs(enacba,z,3);
enacba4 = subs(enacba,z,4);
enacba5 = 1+p1+p2;
% imam 5 enab za 5 neznank (p1,p2,q0,q1,q2)

% teava bi lahko bila le, e bi bile enabe linearno odvisne
% neznanke = {p1,p2,q0,q1,q2};
% enacbe = {enacba1,enacba2,enacba3,enacba4,enacba5};
% matrika_sistema = zeros(5,5);
% for ii = 1:5
%     for jj = 1:5
%         matrika_sistema(ii,jj) = limit(enacbe{ii}/neznanke{jj},neznanke{jj},Inf);
%     end
% end
% 
% det(matrika_sistema)
[pp1,pp2,qq0,qq1,qq2]=solve(enacba1,enacba2,enacba3,enacba4,enacba5,p1,p2,q0,q1,q2); % reim sistem enab

P0 = subs(P,{p1,p2,q0,q1,q2},{pp1,pp2,qq0,qq1,qq2}); % vstavim reitev v P in dobim P0
Q0 = subs(Q,{p1,p2,q0,q1,q2},{pp1,pp2,qq0,qq1,qq2}); % vstavim reitev v Q in dobim Q0

G_P = B/A;     % proces
G_R = Q0/P0;   % regulator (reitev problema pozicionirananja polov)

G_w = simple(G_P*G_R/(1+G_P*G_R)); % prenosna funkcija z reference na regulirano veliino
G_u = simple(G_R/(1+G_P*G_R));     % prenosna funkcija z reference na regulirno veliino

poli_w = eval(solve(1/G_w))          % poli funkcije G_w
nicle_w = eval(solve(G_w))           % nile funkcije G_w
poli_u = eval(solve(1/G_u))          % poli funkcije G_u
nicle_u = eval(solve(G_u))           % nile funkcije G_u

odziv_na_step = iztrans(G_w * z/(z-1),z,k);  % odziv na stopnico
odziv_regulirne = iztrans(G_u * z/(z-1),z,k);  % odziv na stopnico

k_v = 0:40;
figure(1)
subplot(211)
hold on
stem(k_v, subs(odziv_na_step,k,k_v),'r')
xlabel('$$k$$','Interpreter','Latex','FontSize',14);
ylabel('$$y(k)$$','Interpreter','Latex','FontSize',14);
subplot(212)
hold on
stairs(k_v, subs(odziv_regulirne,k,k_v),'r')
xlabel('$$k$$','Interpreter','Latex','FontSize',14);
ylabel('$$u(k)$$','Interpreter','Latex','FontSize',14);
legend('eleni poli','el. poli + I-regulator')
