% Nartovanje splonega linearnega regulatorja za pozicioniranje polov

%% Inicializacija

clc
clear all
close all

% resolucija ekrana
screen_size = get(0, 'ScreenSize');


syms z k p1 p2 p3 q0 q1 q2 q3


% PRAKTIEN PRIMER:
%   G_z(s) = 10/(s+1)(s+2)(s+3)

G_z = zpk([],[-1 -2 -3],10)

%% Kratek as vzorenja - poli v 0

T_1 = 0.01; % Kratek as vzorenja
G_T_1 = tf(c2d(G_z, T_1)); % ZOH diskretizacija

B = poly2sym(G_T_1.num{:},z)/z^3;
A = poly2sym(G_T_1.den{:},z)/z^3;

% zakasnitve ni, d = 0
% stopnja A(z^-1) in B(z^-1) je m = 3
% HOEM E INTEGRIRNO AKCIJO

P = 1 + p1*z^(-1) + p2*z^(-2) + p3*z^(-3);  % stopnja(P) = mi = m+d = 3
Q = q0 + q1*z^(-1) + q2*z^(-2) + q3*z^(-3); % stopnja(A) = ni = m = 3

PAQB = simple(P*A + Q*B);
A_zeleni  = (1)^6;  % stopnja(A_zeleni) = 2m+d = 6 (vsi poli so v koordinatnem izhodiu)

enacba = PAQB-A_zeleni; % ta enaba mora veljati za vse vrednosti z;
% ker iem 7 neznank, potrebujem 7 enab - 6 dobim tako, da vstavim poljubnih 6 neznank, za katere enaba mora veljati
% sedmo enabo dobim tako, da upotevam zahtevo za I-regulator: sum(p_i) = -1
enacba1 = subs(enacba,z,1);
enacba2 = subs(enacba,z,2);
enacba3 = subs(enacba,z,3);
enacba4 = subs(enacba,z,4);
enacba5 = subs(enacba,z,5);
enacba6 = subs(enacba,z,6);
enacba7 = 1+p1+p2+p3;
% imam 7 enab za 7 neznank (p1,p2,p3,q0,q1,q2,q3)

[pp1,pp2,pp3,qq0,qq1,qq2,qq3]=solve(enacba1,enacba2,enacba3,enacba4,enacba5,enacba6,enacba7,p1,p2,p3,q0,q1,q2,q3); % reim sistem enab

P0 = subs(P,{p1,p2,p3,q0,q1,q2,q3},{pp1,pp2,pp3,qq0,qq1,qq2,qq3}); % vstavim reitev v P in dobim P0
Q0 = subs(Q,{p1,p2,p3,q0,q1,q2,q3},{pp1,pp2,pp3,qq0,qq1,qq2,qq3}); % vstavim reitev v Q in dobim Q0

G_P = B/A;     % proces
G_R = Q0/P0;   % regulator (reitev problema pozicionirananja polov)

G_w = simple(G_P*G_R/(1+G_P*G_R)); % prenosna funkcija z reference na regulirano veliino
G_u = simple(G_R/(1+G_P*G_R));     % prenosna funkcija z reference na regulirno veliino

poli_w = eval(solve(1/G_w))          % poli funkcije G_w
nicle_w = eval(solve(G_w))           % nile funkcije G_w
poli_u = eval(solve(1/G_u))          % poli funkcije G_u
nicle_u = eval(solve(G_u))           % nile funkcije G_u

odziv_na_step = iztrans(G_w * z/(z-1),z,k);  % odziv na stopnico
odziv_regulirne = iztrans(G_u * z/(z-1),z,k);  % odziv na stopnico

k_v = 0:80;
figure(1)
subplot(211)
hold on
plot(k_v*T_1, subs(odziv_na_step,k,k_v),'b')
subplot(212)
hold on
stairs(k_v*T_1, subs(odziv_regulirne,k,k_v)/1e6,'b')


%% Dalji as vzorenja - poli v 0

T_2 = 0.1; % Dolg as vzorenja
G_T_2 = tf(c2d(G_z, T_2)); % ZOH diskretizacija

B = poly2sym(G_T_2.num{:},z)/z^3;
A = poly2sym(G_T_2.den{:},z)/z^3;

% zakasnitve ni, d = 0
% stopnja A(z^-1) in B(z^-1) je m = 3
% HOEM E INTEGRIRNO AKCIJO

P = 1 + p1*z^(-1) + p2*z^(-2) + p3*z^(-3);  % stopnja(P) = mi = m+d = 3
Q = q0 + q1*z^(-1) + q2*z^(-2) + q3*z^(-3); % stopnja(A) = ni = m = 3

PAQB = simple(P*A + Q*B);
A_zeleni  = (1)^6;  % stopnja(A_zeleni) = 2m+d = 6 (vsi poli so v koordinatnem izhodiu

enacba = PAQB-A_zeleni; % ta enaba mora veljati za vse vrednosti z;
% ker iem 7 neznank, potrebujem 7 enab - 6 dobim tako, da vstavim poljubnih 6 neznank, za katere enaba mora veljati
% sedmo enabo dobim tako, da upotevam zahtevo za I-regulator: sum(p_i) = -1
enacba1 = subs(enacba,z,1);
enacba2 = subs(enacba,z,2);
enacba3 = subs(enacba,z,3);
enacba4 = subs(enacba,z,4);
enacba5 = subs(enacba,z,5);
enacba6 = subs(enacba,z,6);
enacba7 = 1+p1+p2+p3;
% imam 7 enab za 7 neznank (p1,p2,p3,q0,q1,q2,q3)

[pp1,pp2,pp3,qq0,qq1,qq2,qq3]=solve(enacba1,enacba2,enacba3,enacba4,enacba5,enacba6,enacba7,p1,p2,p3,q0,q1,q2,q3); % reim sistem enab

P0 = subs(P,{p1,p2,p3,q0,q1,q2,q3},{pp1,pp2,pp3,qq0,qq1,qq2,qq3}); % vstavim reitev v P in dobim P0
Q0 = subs(Q,{p1,p2,p3,q0,q1,q2,q3},{pp1,pp2,pp3,qq0,qq1,qq2,qq3}); % vstavim reitev v Q in dobim Q0

G_P = B/A;     % proces
G_R = Q0/P0;   % regulator (reitev problema pozicionirananja polov)

G_w = simple(G_P*G_R/(1+G_P*G_R)); % prenosna funkcija z reference na regulirano veliino
G_u = simple(G_R/(1+G_P*G_R));     % prenosna funkcija z reference na regulirno veliino

poli_w = eval(solve(1/G_w))          % poli funkcije G_w
nicle_w = eval(solve(G_w))           % nile funkcije G_w
poli_u = eval(solve(1/G_u))          % poli funkcije G_u
nicle_u = eval(solve(G_u))           % nile funkcije G_u

odziv_na_step = iztrans(G_w * z/(z-1),z,k);  % odziv na stopnico
odziv_regulirne = iztrans(G_u * z/(z-1),z,k);  % odziv na stopnico

k_v = 0:8;
k_v_resample = 0:80;
T_2_resample = T_2 / 10;
UU = eval(subs(odziv_regulirne,k,k_v))';
UU_resample = reshape(repmat(UU',10,1),[],1); % reshape vzame najprej prvo vrstico, nato drugo...
UU_resample = UU_resample(1:end-9);
YY_resample = lsim(G_z,UU_resample,k_v_resample*T_2_resample,0,'zoh');

figure(1)
set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran
subplot(211)
hold on
plot(k_v_resample*T_2_resample, YY_resample,'r')
subplot(212)
hold on
stairs(k_v*T_2, UU/1000,'r')


%% Zelo dolg as vzorenja - poli v 0

T_3 = 0.5; % Izredno dolg as vzorenja
G_T_3 = tf(c2d(G_z, T_3)); % ZOH diskretizacija

B = poly2sym(G_T_3.num{:},z)/z^3;
A = poly2sym(G_T_3.den{:},z)/z^3;

% zakasnitve ni, d = 0
% stopnja A(z^-1) in B(z^-1) je m = 3
% HOEM E INTEGRIRNO AKCIJO

P = 1 + p1*z^(-1) + p2*z^(-2) + p3*z^(-3);  % stopnja(P) = mi = m+d = 3
Q = q0 + q1*z^(-1) + q2*z^(-2) + q3*z^(-3); % stopnja(A) = ni = m = 3

PAQB = simple(P*A + Q*B);
A_zeleni  = (1)^6;  % stopnja(A_zeleni) = 2m+d = 6 (vsi poli so v koordinatnem izhodiu

enacba = PAQB-A_zeleni; % ta enaba mora veljati za vse vrednosti z;
% ker iem 7 neznank, potrebujem 7 enab - 6 dobim tako, da vstavim poljubnih 6 neznank, za katere enaba mora veljati
% sedmo enabo dobim tako, da upotevam zahtevo za I-regulator: sum(p_i) = -1
enacba1 = subs(enacba,z,1);
enacba2 = subs(enacba,z,2);
enacba3 = subs(enacba,z,3);
enacba4 = subs(enacba,z,4);
enacba5 = subs(enacba,z,5);
enacba6 = subs(enacba,z,6);
enacba7 = 1+p1+p2+p3;
% imam 7 enab za 7 neznank (p1,p2,p3,q0,q1,q2,q3)

[pp1,pp2,pp3,qq0,qq1,qq2,qq3]=solve(enacba1,enacba2,enacba3,enacba4,enacba5,enacba6,enacba7,p1,p2,p3,q0,q1,q2,q3); % reim sistem enab

P0 = subs(P,{p1,p2,p3,q0,q1,q2,q3},{pp1,pp2,pp3,qq0,qq1,qq2,qq3}); % vstavim reitev v P in dobim P0
Q0 = subs(Q,{p1,p2,p3,q0,q1,q2,q3},{pp1,pp2,pp3,qq0,qq1,qq2,qq3}); % vstavim reitev v Q in dobim Q0

G_P = B/A;     % proces
G_R = Q0/P0;   % regulator (reitev problema pozicionirananja polov)

G_w = simple(G_P*G_R/(1+G_P*G_R)); % prenosna funkcija z reference na regulirano veliino
G_u = simple(G_R/(1+G_P*G_R));     % prenosna funkcija z reference na regulirno veliino

poli_w = eval(solve(1/G_w))          % poli funkcije G_w
nicle_w = eval(solve(G_w))           % nile funkcije G_w
poli_u = eval(solve(1/G_u))          % poli funkcije G_u
nicle_u = eval(solve(G_u))           % nile funkcije G_u

odziv_na_step = iztrans(G_w * z/(z-1),z,k);  % odziv na stopnico
odziv_regulirne = iztrans(G_u * z/(z-1),z,k);  % odziv na stopnico

k_v = 0:8;
k_v_resample = 0:80;
T_3_resample = T_3 / 10;
UU = eval(subs(odziv_regulirne,k,k_v))';
UU_resample = reshape(repmat(UU',10,1),[],1); % reshape vzame najprej prvo vrstico, nato drugo...
UU_resample = UU_resample(1:end-9);
YY_resample = lsim(G_z,UU_resample,k_v_resample*T_3_resample,0,'zoh');

figure(1)
subplot(211)
hold on
plot(k_v_resample*T_3_resample, YY_resample,'g')
xlabel('$$t$$','Interpreter','Latex','FontSize',14);
ylabel('$$y(t)$$','Interpreter','Latex','FontSize',14);
legend('T = 0,01','T = 0,1','T = 0,5')
subplot(212)
hold on
stairs(k_v*T_3, UU,'g')
xlabel('$$t$$','Interpreter','Latex','FontSize',14);
ylabel('$$u(t)$$','Interpreter','Latex','FontSize',14);
legend('T = 0,01 (u/10^6)','T = 0,1 (u/10^3)','T = 0,5')


%% Risanje na slike PID

% Odprem slike iz primera 34 - nartovanje PID
openfig('Prim34_sl1.fig','new');
hold on
hh = line(k_v_resample*T_3_resample, YY_resample)
set(hh,'LineWidth',2,'Color',[0 0 0.8]);
axis([0 12 0 2.1])
openfig('Prim34_sl2.fig','new');
hold on
hh = stairs(k_v*T_3, UU);
set(hh,'LineWidth',2,'Color',[0 0 0.8]);


%% Srednji as vzorenja - poli v 0.7

T_2 = 0.1; % Dolg as vzorenja
G_T_2 = tf(c2d(G_z, T_2)); % ZOH diskretizacija

B = poly2sym(G_T_2.num{:},z)/z^3;
A = poly2sym(G_T_2.den{:},z)/z^3;

% zakasnitve ni, d = 0
% stopnja A(z^-1) in B(z^-1) je m = 3
% HOEM E INTEGRIRNO AKCIJO

P = 1 + p1*z^(-1) + p2*z^(-2) + p3*z^(-3);  % stopnja(P) = mi = m+d = 3
Q = q0 + q1*z^(-1) + q2*z^(-2) + q3*z^(-3); % stopnja(A) = ni = m = 3

PAQB = simple(P*A + Q*B);
A_zeleni  = (1-0.7*z^(-1))^6;  % stopnja(A_zeleni) = 2m+d = 6 (vsi poli so v koordinatnem izhodiu

enacba = PAQB-A_zeleni; % ta enaba mora veljati za vse vrednosti z;
% ker iem 7 neznank, potrebujem 7 enab - 6 dobim tako, da vstavim poljubnih 6 neznank, za katere enaba mora veljati
% sedmo enabo dobim tako, da upotevam zahtevo za I-regulator: sum(p_i) = -1
enacba1 = subs(enacba,z,1);
enacba2 = subs(enacba,z,2);
enacba3 = subs(enacba,z,3);
enacba4 = subs(enacba,z,4);
enacba5 = subs(enacba,z,5);
enacba6 = subs(enacba,z,6);
enacba7 = 1+p1+p2+p3;
% imam 7 enab za 7 neznank (p1,p2,p3,q0,q1,q2,q3)

[pp1,pp2,pp3,qq0,qq1,qq2,qq3]=solve(enacba1,enacba2,enacba3,enacba4,enacba5,enacba6,enacba7,p1,p2,p3,q0,q1,q2,q3); % reim sistem enab

P0 = subs(P,{p1,p2,p3,q0,q1,q2,q3},{pp1,pp2,pp3,qq0,qq1,qq2,qq3}); % vstavim reitev v P in dobim P0
Q0 = subs(Q,{p1,p2,p3,q0,q1,q2,q3},{pp1,pp2,pp3,qq0,qq1,qq2,qq3}); % vstavim reitev v Q in dobim Q0

G_P = B/A;     % proces
G_R = Q0/P0;   % regulator (reitev problema pozicionirananja polov)

G_w = simple(G_P*G_R/(1+G_P*G_R)); % prenosna funkcija z reference na regulirano veliino
G_u = simple(G_R/(1+G_P*G_R));     % prenosna funkcija z reference na regulirno veliino

poli_w = eval(solve(1/G_w))          % poli funkcije G_w
nicle_w = eval(solve(G_w))           % nile funkcije G_w
poli_u = eval(solve(1/G_u))          % poli funkcije G_u
nicle_u = eval(solve(G_u))           % nile funkcije G_u

odziv_na_step = iztrans(G_w * z/(z-1),z,k);  % odziv na stopnico
odziv_regulirne = iztrans(G_u * z/(z-1),z,k);  % odziv na stopnico

k_v = 0:120;
k_v_resample = 0:1200;
T_2_resample = T_2 / 10;
UU = eval(subs(odziv_regulirne,k,k_v))';
UU_resample = reshape(repmat(UU',10,1),[],1); % reshape vzame najprej prvo vrstico, nato drugo...
UU_resample = UU_resample(1:end-9);
YY_resample = lsim(G_z,UU_resample,k_v_resample*T_2_resample,0,'zoh');

figure(2)
hh = line(k_v_resample*T_2_resample, YY_resample);
set(hh,'LineWidth',2,'Color',[0 0.8 0]);
legend('zvezni PID_{id}','zvezni PID_{real}','disk PID_{zd1}','disk PID_{Tus1}','disk PID_{zd2}','disk PID_{Tus2}','disk PID_{Tus2real}','poli v 0','poli v 0.7')
figure(3)
hh = stairs(k_v*T_2, UU);
set(hh,'LineWidth',2,'Color',[0 0.8 0]);
legend('zvezni PID_{id}','zvezni PID_{real}','disk PID_{zd1}','disk PID_{zd2}','disk PID_{Tus2}','disk PID_{Tus2real}','poli v 0','poli v 0.7')
