clear all
close all
clc

% Namen tega primera je predstaviti vodljivost sistemov ter njihovo pretvorbo v kanonine oblike
A=[0.1 0.6;-0.1 -0.4];

% Vzemimo nevodljiv sistem ...
b1=[0.3;-0.1];
ct=[1 1];
sys1=ss(A,b1,ct,0,-1);
% ... in % vodljiv sistem
b2=[0.3; 0.1];
sys2=ss(A,b2,ct,0,-1);

% Izraunamo matriko vodljivosti za nevodljiv ...
Qv1=ctrb(sys1);
detQv1 = det(Qv1)
rangQv1 = rank([b1, A*b1]) % [b, A*b] je enako kot Qv
% ... in vodljiv sistem
Qv2=ctrb(sys2);
detQv2 = det(Qv2)
rangQv2 = rank([b2, A*b2]) % [b, A*b] je enako kot Qv

pause

% Pretvorimo v vodljivostno kanonino obliko, ki pa ni regulatorska
% (gre za obliko VTKO - glej ubenik Matko: Diskretni regulacijski sistemi)
% Je pa res, da je pogoj za obstoj pretvorbo enak - nesingularna vodljivostna matrika
disp([char(13) char(13)]) % gre v novo vrstico - vertikalni presledek
try
    disp('nevodljiv sistem v VTKO')
    sys1_VTKO = canon(sys1,'companion')
catch razlog
    disp(['koda napake: ',razlog.identifier])
    disp(['sistem je nevodljiv, det(Qv1) = ',num2str(det(Qv1))])
end
disp([char(13) char(13)])
try
    disp('vodljiv sistem v VTKO')
    sys2_VTKO = canon(sys2,'companion')
catch razlog
    disp(['koda napake: ',razlog.identifier])
    disp(['sistem je nevodljiv, det(Qv2) = ',num2str(det(Qv2))])
end

pause

% Pretvorba v vodljivostno regulatorsko kanonino obliko s transformacijsko matriko Qv*W
disp([char(13) char(13) 'Pretvorba vodljivega sistema v vodljivostno regulatorsko kanonino obliko s transformacijsko matriko Qv*W'])
kar_pol = poly(A); % karakteristini polinom (od leve proti desni imamo: a_0=1, a_1, ... a_n)
% za matriko W potrebujemo od zgoraj navzdol: (a_n), a_{n-1}, ..., a_1, a_0=1 - prvi je v oklepaju, ker ga za W zares ne potrebujemo
kar_pol = flipud(kar_pol'); % koeficiente torej transponiramo v stolpec in flipnemo
for jj = 1:length(A) % pri kvadratni matriki je length kar dimenzija
    W(:,jj) = [kar_pol(jj+1:end); zeros(jj-1,1)];
end
W
T = Qv2*W
A_v = inv(T)*A*T
b_v = inv(T)*b2
ct_v = ct*T

pause

% Pretvorimo v diagonalno obliko nevodljiv ...
disp([char(13) char(13) 'Diagonalna kanonina oblika za nevodljiv sistem'])
sys1_diagonal = canon(sys1,'modal')
pause
% ... in vodljiv sistem
disp([char(13) 'Diagonalna kanonina oblika za vodljiv sistem'])
sys2_diagonal = canon(sys2,'modal')

pause

%Pretvorimo v prenosno funkcijo (tf) in obliko zpk ter e minimalno realizacijo nevodljiv ...
disp([char(13) char(13) 'Prenosna funkcija za nevodljiv sistem'])
PF1 = tf(sys1)
zpk(PF1)
minreal(PF1)

pause

% ... in vodljiv sistem
disp([char(13) 'Prenosna funkcija za vodljiv sistem'])
PF2 = tf(sys2)
zpk(PF2)

pause

figure
% resolucija ekrana
screen_size = get(0, 'ScreenSize');
set(gcf, 'Position', screen_size); % poveamo graf ez cel ekran

U = 10*randn(100,1);

hh1 = title('Najprej bomo prikazali odziv nevodljivega sistema, e je zaetno stanje kolinearno vhodnemu vektorju.');
set(hh1,'Color',[0 0.6 0],'Fontsize',14)
hh2 = xlabel('Pritisni tipko.');
set(hh2,'Color',[1 0 0],'Fontsize',14)
hold on
pause

[Y,t,X] = lsim(sys1,U,[],20*b1);
plot(X(:,1),X(:,2),'b*-')

set(hh1, 'String', 'Sedaj bomo prikazali odziv nevodljivega sistema, e zaetno stanje ni kolinearno vhodnemu vektorju.');
pause

[Y,t,X] = lsim(sys1,U,[],[3;2]);
plot(X(:,1),X(:,2),'g*-')

set(hh1, 'String', 'Sedaj bomo prikazali odziv vodljivega sistema.');
pause

[Y,t,X] = lsim(sys2,U,[],[2;0]);
plot(X(:,1),X(:,2),'r*-')
set(hh2, 'String', '');
